# -*- coding: UTF-8 -*-

__revision__ = '$Id: preferences.py 1597 2011-10-04 18:41:24Z piotrek $'

# Copyright (c) 2005-2009 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging
import os
import gtk

import db
import gutils
import initialize
import plugins.extensions

try:
    import gtkspell
    spell_support = 1
except:
    spell_support = 0

log = logging.getLogger("Griffith")


def show_preferences(self, page=None):
    w = self.widgets['preferences']
    # number
    if self.config.get('number', True, section='mainlist') == False:
        w['view_number'].set_active(False)
    else:
        w['view_number'].set_active(True)
    # image
    if self.config.get('image', True, section='mainlist') == False:
        w['view_image'].set_active(False)
    else:
        w['view_image'].set_active(True)
    # original title
    if self.config.get('otitle', True, section='mainlist') == False:
        w['view_o_title'].set_active(False)
    else:
        w['view_o_title'].set_active(True)
    # title
    if self.config.get('title', True, section='mainlist') == False:
        w['view_title'].set_active(False)
    else:
        w['view_title'].set_active(True)
    # director
    if self.config.get('director', True, 'mainlist') == False:
        w['view_director'].set_active(False)
    else:
        w['view_director'].set_active(True)
    # genre
    if self.config.get('genre', True, 'mainlist') == False:
        w['view_genre'].set_active(False)
    else:
        w['view_genre'].set_active(True)
    # seen
    if self.config.get('seen', True, 'mainlist') == False:
        w['view_seen'].set_active(False)
    else:
        w['view_seen'].set_active(True)
    # year
    if self.config.get('year', True, 'mainlist') == False:
        w['view_year'].set_active(False)
    else:
        w['view_year'].set_active(True)
    # runtime
    if self.config.get('runtime', True, 'mainlist') == False:
        w['view_runtime'].set_active(False)
    else:
        w['view_runtime'].set_active(True)
    # rating
    if self.config.get('rating', True, 'mainlist') == False:
        w['view_rating'].set_active(False)
    else:
        w['view_rating'].set_active(True)
    # created
    if self.config.get('created', True, 'mainlist') == False:
        w['view_created'].set_active(False)
    else:
        w['view_created'].set_active(True)
    # updated
    if self.config.get('updated', True, 'mainlist') == False:
        w['view_updated'].set_active(False)
    else:
        w['view_updated'].set_active(True)

    # email reminder
    if self.config.get('use_auth', False, section='mail') == False:
        w['mail_use_auth'].set_active(False)
    else:
        w['mail_use_auth'].set_active(True)

    if self.config.get('mail_use_tls', False, section='mail') == False:
        w['mail_use_tls'].set_active(False)
    else:
        w['mail_use_tls'].set_active(True)

    w['mail_smtp_server'].set_text(self.config.get('smtp_server', 'localhost', section='mail'))
    w['mail_smtp_port'].set_text(self.config.get('mail_smtp_port', '25', section='mail'))
    w['mail_username'].set_text(self.config.get('username', '', section='mail'))
    w['mail_password'].set_text(self.config.get('password', '', section='mail'))
    w['mail_email'].set_text(self.config.get('email', 'griffith@localhost', section='mail'))

    # pdf reader
    w['epdf_reader'].set_text(self.pdf_reader)

    # pdf font
    if self.config.get('font'):
        w['font'].set_filename(self.config.get('font'))
    if self.config.get('font_size'):
        try:
            w['font_size'].set_value(float(self.config.get('font_size')))
        except:
            w['font_size'].set_value(18.0)

    # pdf elements
    pdf_elements = self.config.get('pdf_elements', 'image,director,genre,cast')
    for pdf_element in pdf_elements.split(','):
        if 'pdf_' + pdf_element.strip() in w:
            w['pdf_' + pdf_element.strip()].set_active(True)

    # defaults (for static data only)
    w['condition'].set_active( gutils.digits_only(self.config.get('condition', 0, section='defaults'), 5) )
    w['region'].set_active( gutils.digits_only(self.config.get('region', 0, section='defaults'), 8) )
    w['layers'].set_active( gutils.digits_only(self.config.get('layers', 0, section='defaults'), 4) )
    w['color'].set_active( gutils.digits_only(self.config.get('color', 0, section='defaults'), 3 ))
    tmp = self.config.get('media', 0, section='defaults')
    if tmp is not None and int(tmp) in self.media_ids:
        if int(self.config.get('media', 0, section='defaults')) > 0:
            w['media'].set_active( gutils.findKey(int(self.config.get('media', 0, section='defaults')), self.media_ids) )
        else:
            w['media'].set_active(0)
    tmp = self.config.get('vcodec', 0, section='defaults')
    if tmp is not None and int(tmp) in self.vcodecs_ids:
        if int(self.config.get('vcodec', 0, section='defaults')) > 0:
            w['vcodec'].set_active(    int(gutils.findKey(int(self.config.get('vcodec', 0, section='defaults')), self.vcodecs_ids)) )
        else:
            w['vcodec'].set_active(0)
    w['seen'].set_active(bool(self.config.get('seen', True, section='defaults')))

    # search for:
    w['s_classification'].set_active(bool(self.config.get('s_classification', True, section='add')))
    w['s_country'].set_active(bool(self.config.get('s_country', True, section='add')))
    w['s_director'].set_active(bool(self.config.get('s_director', True, section='add')))
    w['s_genre'].set_active(bool(self.config.get('s_genre', True, section='add')))
    w['s_image'].set_active(bool(self.config.get('s_image', True, section='add')))
    w['s_notes'].set_active(bool(self.config.get('s_notes', True, section='add')))
    w['s_o_site'].set_active(bool(self.config.get('s_o_site', True, section='add')))
    w['s_o_title'].set_active(bool(self.config.get('s_o_title', True, section='add')))
    w['s_plot'].set_active(bool(self.config.get('s_plot', True, section='add')))
    w['s_rating'].set_active(bool(self.config.get('s_rating', True, section='add')))
    w['s_runtime'].set_active(bool(self.config.get('s_runtime', True, section='add')))
    w['s_site'].set_active(bool(self.config.get('s_site', True, section='add')))
    w['s_studio'].set_active(bool(self.config.get('s_studio', True, section='add')))
    w['s_title'].set_active(bool(self.config.get('s_title', True, section='add')))
    w['s_trailer'].set_active(bool(self.config.get('s_trailer', True, section='add')))
    w['s_cast'].set_active(bool(self.config.get('s_cast', True, section='add')))
    w['s_year'].set_active(bool(self.config.get('s_year', True, section='add')))
    w['s_screenplay'].set_active(bool(self.config.get('s_screenplay', True, section='add')))
    w['s_cameraman'].set_active(bool(self.config.get('s_cameraman', True, section='add')))
    w['s_resolution'].set_active(bool(self.config.get('s_resolution', True, section='add')))
    w['s_barcode'].set_active(bool(self.config.get('s_barcode', True, section='add')))

    if self.config.get('sortby', section='mainlist'):
        tmp = self.sort_criteria.index(self.config.get('sortby', section='mainlist'))
        w['sortby'].set_active(tmp)
    w['sortby_reverse'].set_active(bool(self.config.get('sortby_reverse', False, section='mainlist')))

    w['s_limit'].set_value(gutils.digits_only(self.config.get('limit', 0, section='mainlist')))

    plugins = gutils.read_plugins('PluginMovie', \
        self.locations['movie_plugins'])
    plugins.sort()
    mcounter = 0
    default_movie_plugin = self.config.get('default_movie_plugin')
    for p in plugins:
        plugin_module = os.path.basename(p).replace('.py','')
        plugin_name = plugin_module.replace('PluginMovie','')
        if plugin_name == default_movie_plugin:
            w['default_plugin'].set_active(mcounter)
        mcounter = mcounter + 1

    # rating image
    try:
        rimage = int(self.config.get('rating_image', 0))
    except:
        rimage = 0
    w['rating_image'].set_active(rimage)

    # spellchecker
    if self.config.get('gtkspell', False, section='spell') == False:
        w['spellchecker'].set_active(False)
    else:
        w['spellchecker'].set_active(True)
    self.on_cb_spellchecker_pref_toggled(w['spellchecker'])

    if self.config.get('notes', True, section='spell') == False:
        w['spell_notes'].set_active(False)
    else:
        w['spell_notes'].set_active(True)

    if self.config.get('plot', True, section='spell') == False:
        w['spell_plot'].set_active(False)
    else:
        w['spell_plot'].set_active(True)

    w['spell_lang'].set_text(str(self.config.get('lang', 'en', section='spell')))

    if page is not None:
        w['notebook'].set_current_page(page)
    w['window'].show()

def save_preferences(self):
    w = self.widgets['preferences']
    c = self.config
    global spell_support

    was_false = notes_was_false = plot_was_false = 1

    if c.get('gtkspell', False, section='spell') == True:
        was_false = 0

    if c.get('notes', False, section='spell') == True:
        notes_was_false = 0

    if c.get('plot', False, section='spell') == True:
        plot_was_false = 0

    # number
    if w['view_number'].get_active():
        c.set('number', 'True', section='mainlist')
    else:
        c.set('number', 'False', section='mainlist')
    # image
    if w['view_image'].get_active():
        c.set('image', 'True', section='mainlist')
    else:
        c.set('image', 'False', section='mainlist')
    # original title
    if w['view_o_title'].get_active():
        c.set('otitle', 'True', section='mainlist')
    else:
        c.set('otitle', 'False', section='mainlist')
    # title
    if w['view_title'].get_active():
        c.set('title', 'True', section='mainlist')
    else:
        c.set('title', 'False', section='mainlist')
    # director
    if w['view_director'].get_active():
        c.set('director', 'True', section='mainlist')
    else:
        c.set('director', 'False', section='mainlist')
    # genre
    if w['view_genre'].get_active():
        c.set('genre', 'True', section='mainlist')
    else:
        c.set('genre', 'False', section='mainlist')
    # seen
    if w['view_seen'].get_active():
        c.set('seen', 'True', section='mainlist')
    else:
        c.set('seen', 'False', section='mainlist')
    # year
    if w['view_year'].get_active():
        c.set('year', 'True', section='mainlist')
    else:
        c.set('year', 'False', section='mainlist')
    # runtime
    if w['view_runtime'].get_active():
        c.set('runtime', 'True', section='mainlist')
    else:
        c.set('runtime', 'False', section='mainlist')
    # rating
    if w['view_rating'].get_active():
        c.set('rating', 'True', section='mainlist')
    else:
        c.set('rating', 'False', section='mainlist')
    # created
    if w['view_created'].get_active():
        c.set('created', 'True', section='mainlist')
    else:
        c.set('created', 'False', section='mainlist')
    # updated
    if w['view_updated'].get_active():
        c.set('updated', 'True', section='mainlist')
    else:
        c.set('updated', 'False', section='mainlist')

    # sortby
    if w['sortby'].get_active():
        field = self.sort_criteria[w['sortby'].get_active()]
        if field:
            c.set('sortby', field, section='mainlist')
    else:
        c.set('sortby', 'number', section='mainlist')
    c.set('sortby_reverse', w['sortby_reverse'].get_active(), section='mainlist')

    c.set('limit', str(int(w['s_limit'].get_value())), section='mainlist')


    # pdf font
    if w['font'].get_filename():
        c['font'] = w['font'].get_filename()
    c['font_size'] = int(w['font_size'].get_value())

    # pdf elements
    pdf_elements = ''
    for child in w['pdf_elements_table']:
        if child.get_active():
            pdf_elements = pdf_elements + child.get_name()[4:] + ','
    if pdf_elements:
        c.set('pdf_elements', pdf_elements[:-1])
    else:
        c.set('pdf_elements', pdf_elements)

    # spellchecker
    if w['spellchecker'].get_active():
        c.set('gtkspell', True, section='spell')
    else:
        c.set('gtkspell', False, section='spell')
    if w['spell_notes'].get_active():
        c.set('notes', True, section='spell')
    else:
        c.set('notes', False, section='spell')
    if w['spell_plot'].get_active():
        c.set('plot', True, section='spell')
    else:
        c.set('plot', False, section='spell')

    # rating image
    c['rating_image'] = str(w['rating_image'].get_active())

    #defaults
    media_id = self.media_ids[w['media'].get_active()]
    if media_id is None:
        media_id = 0
    c.set('media', media_id, section='defaults')
    vcodec_id = self.vcodecs_ids[w['vcodec'].get_active()]
    if vcodec_id is None:
        vcodec_id = 0
    c.set('vcodec', vcodec_id, section='defaults')
    c.set('condition', str(w['condition'].get_active()), section='defaults')
    c.set('region', str(w['region'].get_active()), section='defaults')
    c.set('layers', str(w['layers'].get_active()), section='defaults')
    c.set('color', str(w['color'].get_active()), section='defaults')
    c.set('seen', str(w['seen'].get_active()), section='defaults')

    # email reminder
    if w['mail_use_auth'].get_active():
        c.set('use_auth', True, section='mail')
    else:
        c.set('use_auth', False, section='mail')

    if w['mail_use_tls'].get_active():
        c.set('mail_use_tls', True, section='mail')
    else:
        c.set('mail_use_tls', False, section='mail')

    c.set('smtp_server', w['mail_smtp_server'].get_text(), section='mail')
    c.set('mail_smtp_port', w['mail_smtp_port'].get_text(), section='mail')

    c.set('username', w['mail_username'].get_text(), section='mail')
    c.set('password', w['mail_password'].get_text(), section='mail')
    c.set('email', w['mail_email'].get_text(), section='mail')

    # default movie plugin
    if w['default_plugin'].get_active():
        c['default_movie_plugin'] = \
            gutils.on_combo_box_entry_changed(w['default_plugin'])
    # search for:
    c.set('s_classification', w['s_classification'].get_active(), section='add')
    c.set('s_country', w['s_country'].get_active(), section='add')
    c.set('s_director', w['s_director'].get_active(), section='add')
    c.set('s_genre', w['s_genre'].get_active(), section='add')
    c.set('s_image', w['s_image'].get_active(), section='add')
    c.set('s_notes', w['s_notes'].get_active(), section='add')
    c.set('s_o_site', w['s_o_site'].get_active(), section='add')
    c.set('s_o_title', w['s_o_title'].get_active(), section='add')
    c.set('s_plot', w['s_plot'].get_active(), section='add')
    c.set('s_rating', w['s_rating'].get_active(), section='add')
    c.set('s_runtime', w['s_runtime'].get_active(), section='add')
    c.set('s_site', w['s_site'].get_active(), section='add')
    c.set('s_studio', w['s_studio'].get_active(), section='add')
    c.set('s_title', w['s_title'].get_active(), section='add')
    c.set('s_trailer', w['s_trailer'].get_active(), section='add')
    c.set('s_cast', w['s_cast'].get_active(), section='add')
    c.set('s_year', w['s_year'].get_active(), section='add')
    c.set('s_screenplay', w['s_screenplay'].get_active(), section='add')
    c.set('s_cameraman', w['s_cameraman'].get_active(), section='add')
    c.set('s_resolution', w['s_resolution'].get_active(), section='add')
    c.set('s_barcode', w['s_barcode'].get_active(), section='add')

    mcounter = 0
    for p in self.plugins:
        plugin_module = os.path.basename(p).replace('.py','')
        plugin_name = plugin_module.replace('PluginMovie','')
        if gutils.on_combo_box_entry_changed(w['default_plugin']) == plugin_name:
            break
        mcounter = mcounter + 1
    self.widgets['add']['source'].set_active(mcounter)

    save_reader = w['epdf_reader'].get_text()

    c.set('lang', w['spell_lang'].get_text(), section='spell')
    c['pdf_reader'] = save_reader

    if spell_support:
        if c.get('gtkspell', False, section='spell') == False and not was_false:
            self.notes_spell.detach()
            self.plot_spell.detach()
        elif c.get('gtkspell', False, section='spell') == True and was_false:
            initialize.spellcheck(self)
        else:
            pass

        if c.get('gtkspell', False, section='spell') == True:
            if c.get('plot', True, section='spell') == False and not plot_was_false:
                self.plot_spell.detach()
            elif c.get('plot', True, section='spell') == True and plot_was_false:
                self.plot_spell = gtkspell.Spell(self.widgets['add']['plot'])
                self.plot_spell.set_language(c.get('lang', 'en', section='spell'))
            else:
                pass

            if c.get('notes', True, section='spell') == False and not notes_was_false:
                self.notes_spell.detach()
            elif c.get('notes', True, section='spell') == True and notes_was_false:
                self.notes_spell = gtkspell.Spell(self.widgets['add']['notes'])
                self.notes_spell.set_language(c.get('lang', 'en', section='spell'))
            else:
                pass
    self.pdf_reader = save_reader

    # extensions settings
    for ext_name in plugins.extensions.by_name:
        preferenceswidgets = plugins.extensions.by_name[ext_name].preferenceswidgets
        for prefname in preferenceswidgets:
            widget = preferenceswidgets[prefname]
            if isinstance(widget, gtk.CheckButton):
                value = widget.get_active()
            elif isinstance(widget, gtk.Entry):
                value = widget.get_text()
            elif isinstance(widget, gtk.ComboBox):
                iter = widget.get_active_iter()
                if iter:
                    value = widget.get_model().get_value(iter, 1)
            else:
                log.error('widget type not supported %s', type(widget))
                continue
            c.set("%s_%s" % (ext_name, prefname), value, section='extensions')

    # database
    old = c.to_dict(section='database')

    c.set('host', w['db_host'].get_text(), section='database')
    c.set('port', int(w['db_port'].get_value()), section='database')
    c.set('name', w['db_name'].get_text(), section='database')
    c.set('user', w['db_user'].get_text(), section='database')
    c.set('passwd', w['db_passwd'].get_text(), section='database')
    db_type = int(w['db_type'].get_active())
    if db_type == 1:
        c.set('type', 'postgres', section='database')
    elif db_type == 2:
        c.set('type', 'mysql', section='database')
    elif db_type == 3:
        c.set('type', 'mssql', section='database')
    else:
        c.set('type', 'sqlite', section='database')

    if old['type'] != c.get('type', section='database') or (old['type']!='sqlite' and (\
            old['host'] != c.get('host', section='database') or \
            old['port'] != c.get('port', section='database') or \
            old['user'] != c.get('user', section='database') or \
            old['passwd'] != c.get('passwd', section='database'))) or \
            old['name'] != c.get('name', section='database'):
        log.info('DATABASE: connecting to new db server...')
        import sql
        from sqlalchemy.exc import InvalidRequestError
        from initialize import dictionaries, people_treeview

        # new database connection
        self.initialized = False
        if c.has_key('posters'):
            c['posters'] = None # force update
        try:
            self.db.dispose()
            self.db = sql.GriffithSQL(c, self.locations['home'], fallback=True)
        except InvalidRequestError, e:
            log.exception('')
            c.set('type', 'sqlite', section='database')
            w['db_type'].set_active(0)
            self.db = sql.GriffithSQL(c, self.locations['home'])

        log.info("New database Engine: %s" % self.db.session.bind.engine.name)

        # initialize new database
        self.total = int(self.db.session.query(db.Movie).count())
        self.count_statusbar()
        dictionaries(self)
        people_treeview(self, False)
        self.initialized = True
    self.clear_details()
    self.filter_txt_forced()
    c.save()

    # reload extensions
    initialize.extensions(self)



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2006-2009
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string
import amazon
import threading
import gtk
from operator import isSequenceType
from urlparse import urlsplit
import logging
log = logging.getLogger("Griffith")

plugin_name         = "Amazon"
plugin_description  = "Amazon"
plugin_url          = "www.amazon.com/.uk/.de/.ca/.fr/.jp"
plugin_language     = _("International")
plugin_author       = "Michael Jahn"
plugin_author_email = "<mikej06@hotmail.com>"
plugin_version      = "1.2"


class Plugin(movie.Movie):

    def __init__(self, id):
        self.encode   = 'utf8'
        self.movie_id = id
        self.url      = 'http://www.amazon.de/dp/' + str(self.movie_id)

    def open_page(self, parent_window=None, url=None):
        # dont use base functionality
        # use the Amazon Web API
        self.parent_window = parent_window
        try:
            accesskey = self.config.get('amazon_accesskey', None, section='extensions')
            secretkey = self.config.get('amazon_secretkey', None, section='extensions')
            if not accesskey or not secretkey:
                gutils.error(_('Please configure your Amazon Access Key ID and Secret Key correctly in the preferences dialog.'))
                return False
            amazon.setLicense(accesskey, secretkey)

            locale = self.config.get('amazon_locale', 0, section='extensions')
            if locale == '1' or locale == 'UK':
                locale = 'uk'
            elif locale == '2' or locale == 'DE':
                locale = 'de'
            elif locale == '3' or locale == 'CA':
                locale = 'ca'
            elif locale == '4' or locale == 'FR':
                locale = 'fr'
            elif locale == '5' or locale == 'JP':
                locale = 'jp'
            else:
                locale = None
            retriever = AmazonRetriever(self.movie_id, locale, parent_window, self.progress, 'Get')
            retriever.start()
            while retriever.isAlive():
                self.progress.pulse()
                while gtk.events_pending():
                    gtk.main_iteration()
            self.page = retriever.result.Item[0]
        except:
            self.page = ''
            try:
                log.exception('Error retrieving results from amazon.')
                log.error(retriever.result.Request.Errors.Error.Message)
            except:
                pass
        return self.page

    def get_image(self):
        self.image_url = ''
        if hasattr(self.page, 'LargeImage'):
            self.image_url = self.page.LargeImage.URL
        elif hasattr(self.page, 'MediumImage'):
            self.image_url = self.page.MediumImage.URL
        elif hasattr(self.page, 'SmallImage'):
            self.image_url = self.page.SmallImage.URL

    def get_o_title(self):
        self.o_title = ''
        if hasattr(self.page, 'ItemAttributes'):
            if hasattr(self.page.ItemAttributes, 'Title'):
                self.o_title = self.page.ItemAttributes.Title

    def get_title(self):
        self.title = ''
        if hasattr(self.page, 'ItemAttributes'):
            if hasattr(self.page.ItemAttributes, 'Title'):
                self.title = self.page.ItemAttributes.Title

    def get_director(self):
        self.director = ''
        if hasattr(self.page, 'ItemAttributes'):
            if hasattr(self.page.ItemAttributes, 'Director'):
                if isinstance(self.page.ItemAttributes.Director, list):
                    self.director = string.join(self.page.ItemAttributes.Director, ', ')
                else:
                    self.director = self.page.ItemAttributes.Director

    def get_plot(self):
        self.plot = ''
        if hasattr(self.page, 'EditorialReviews'):
            if hasattr(self.page.EditorialReviews, 'EditorialReview'):
                if isSequenceType(self.page.EditorialReviews.EditorialReview):
                    for review in self.page.EditorialReviews.EditorialReview:
                        if string.find(review.Source, 'Amazon') > -1:
                            self.plot = review.Content
                else:
                    if hasattr(self.page.EditorialReviews.EditorialReview, 'Source') and \
                        hasattr(self.page.EditorialReviews.EditorialReview, 'Content') and \
                        string.find(self.page.EditorialReviews.EditorialReview.Source, 'Amazon') > -1:
                        self.plot = self.page.EditorialReviews.EditorialReview.Content

    def get_year(self):
        self.year = ''
        if hasattr(self.page, 'ItemAttributes'):
            if hasattr(self.page.ItemAttributes, 'TheatricalReleaseDate'):
                self.year = self.page.ItemAttributes.TheatricalReleaseDate[:4]
            elif hasattr(self.page.ItemAttributes, 'ReleaseDate'):
                self.year = self.page.ItemAttributes.ReleaseDate[:4]

    def get_runtime(self):
        self.runtime = ''
        if hasattr(self.page, 'ItemAttributes'):
            if hasattr(self.page.ItemAttributes, 'RunningTime'):
                self.runtime = self.page.ItemAttributes.RunningTime

    def get_genre(self):
        # BrowseNodeId 547664 (Genres)
        self.genre = ''
        delimiter = ''
        if hasattr(self.page, 'BrowseNodes') and hasattr(self.page.BrowseNodes, 'BrowseNode'):
            if isSequenceType(self.page.BrowseNodes.BrowseNode):
                for node in self.page.BrowseNodes.BrowseNode:
                    parentnode = node
                    while hasattr(parentnode, 'Ancestors') and parentnode.BrowseNodeId <> '547664' \
                            and parentnode.BrowseNodeId <> '13628901': # no production countries; they are also arranged under genres
                        parentnode = parentnode.Ancestors.BrowseNode
                    if parentnode.BrowseNodeId == '547664':
                        self.genre = self.genre + delimiter + node.Name
                        delimiter = ', '

    def get_cast(self):
        self.cast = ''
        if hasattr(self.page, 'ItemAttributes'):
            if hasattr(self.page.ItemAttributes, 'Actor'):
                if isinstance(self.page.ItemAttributes.Actor, list):
                    self.cast = string.join(self.page.ItemAttributes.Actor, '\n')
                else:
                    self.cast = self.page.ItemAttributes.Actor

    def get_classification(self):
        self.classification = ''
        if hasattr(self.page, 'ItemAttributes'):
            if hasattr(self.page.ItemAttributes, 'AudienceRating'):
                self.classification = self.page.ItemAttributes.AudienceRating

    def get_studio(self):
        self.studio = ''
        if hasattr(self.page, 'ItemAttributes'):
            if hasattr(self.page.ItemAttributes, 'Studio'):
                self.studio = self.page.ItemAttributes.Studio

    def get_o_site(self):
        self.o_site = ''

    def get_site(self):
        if hasattr(self.page, 'DetailPageURL'):
            parts = urlsplit(self.page.DetailPageURL)
            self.site = parts[0] + '://' + parts[1] + '/dp/' + self.movie_id
        else:
            self.site = ''

    def get_trailer(self):
        self.trailer = ''

    def get_country(self):
        # BrowseNodeId 13628901 (production countries)
        self.country = ''
        delimiter = ''
        if hasattr(self.page, 'BrowseNodes') and hasattr(self.page.BrowseNodes, 'BrowseNode'):
            if isSequenceType(self.page.BrowseNodes.BrowseNode):
                for node in self.page.BrowseNodes.BrowseNode:
                    parentnode = node
                    while hasattr(parentnode, 'Ancestors') and parentnode.BrowseNodeId <> '13628901':
                        parentnode = parentnode.Ancestors.BrowseNode
                    if parentnode.BrowseNodeId == '13628901':
                        self.country = self.country + delimiter + node.Name
                        delimiter = ', '

    def get_rating(self):
        self.rating = '0'
        if hasattr(self.page, 'CustomerReviews') and \
            hasattr(self.page.CustomerReviews, 'AverageRating'):
            try:
                tmp_float = float(self.page.CustomerReviews.AverageRating)
                tmp_float = round(2 * tmp_float, 0)
                self.rating = str(tmp_float)
            except:
                pass

    def get_notes(self):
        self.notes = ''
        if hasattr(self.page, 'ItemAttributes'):
            if hasattr(self.page.ItemAttributes, 'EAN'):
                self.notes = 'EAN: ' + self.page.ItemAttributes.EAN

    def get_barcode(self):
        self.barcode = ''
        if hasattr(self.page, 'ItemAttributes'):
            if hasattr(self.page.ItemAttributes, 'EAN'):
                self.barcode = self.page.ItemAttributes.EAN


class SearchPlugin(movie.SearchMovie):

    def __init__(self):
        self.original_url_search   = 'http://www.amazon.de'
        self.translated_url_search = 'http://www.amazon.de'
        self.encode                = 'utf8'
        self.remove_accents        = False

    def search(self, parent_window):
        # dont use base functionality
        # use the Amazon Web API
        self.titles = [""]
        self.ids = [""]
        try:
            accesskey = self.config.get('amazon_accesskey', None, section='extensions')
            secretkey = self.config.get('amazon_secretkey', None, section='extensions')
            if not accesskey or not secretkey:
                gutils.error(_('Please configure your Amazon Access Key ID and Secret Key correctly in the preferences dialog.'))
                return False
            amazon.setLicense(accesskey, secretkey)

            locale = self.config.get('amazon_locale', 0, section='extensions')
            if locale == '1' or locale == 'UK':
                locale = 'uk'
            elif locale == '2' or locale == 'DE':
                locale = 'de'
            elif locale == '3' or locale == 'CA':
                locale = 'ca'
            elif locale == '4' or locale == 'FR':
                locale = 'fr'
            elif locale == '5' or locale == 'JP':
                locale = 'jp'
            else:
                locale = None
            retriever = AmazonRetriever(self.title, locale, parent_window, self.progress)
            retriever.start()
            while retriever.isAlive():
                self.progress.pulse()
                while gtk.events_pending():
                    gtk.main_iteration()
            self.page = retriever.result
        except:
            try:
                log.exception('Error retrieving results from amazon.')
                log.error(retriever.result.Request.Errors.Error.Message)
            except:
                pass
        return self.page

    def get_searches(self):
        if self.page:
            for result in self.page:
                if hasattr(result, 'Item'):
                    if hasattr(result.Item, 'ASIN'):
                        self.add_item(result.Item)
                    else:
                        for item in result.Item:
                            self.add_item(item)
                elif hasattr(result, 'Items'):
                    if hasattr(result.Item, 'ASIN'):
                        self.add_item(result.Items)
                    else:
                        for item in result.Items:
                            self.add_item(item)

    def add_item(self, item):
        self.ids.append(item.ASIN)
        if hasattr(item.ItemAttributes, 'ProductGroup'):
            productGroup = item.ItemAttributes.ProductGroup + ' - '
        else:
            productGroup = ''
        if hasattr(item.ItemAttributes, 'Title'):
            title = item.ItemAttributes.Title
        else:
            title = ''
        if hasattr(item.ItemAttributes, 'TheatricalReleaseDate'):
            theatricalReleaseDate = ' (' + item.ItemAttributes.TheatricalReleaseDate + ')'
        else:
            theatricalReleaseDate = ''
        self.titles.append("%s%s%s (ASIN: %s)" % (productGroup, title, theatricalReleaseDate, item.ASIN))


class AmazonRetriever(threading.Thread):

    def __init__(self, title, locale, parent_window, progress, lookuptype='Search', destination=None):
        self.title = title
        self.locale = locale
        self.result = None
        self.destination = destination
        self.parent_window = parent_window
        self.progress = progress
        # Search or Get
        self.lookuptype = lookuptype
        self._stopevent = threading.Event()
        self._sleepperiod = 1.0
        threading.Thread.__init__(self, name='Retriever')

    def run(self):
        if self.lookuptype == 'Get':
            self.run_get()
        else:
            self.run_search()

    def run_search(self):
        self.result = []
        try:
            try:
                tmp = amazon.searchByTitle(self.title, type='ItemAttributes', product_line='Video', locale=self.locale, page=1)
                self.result.append(tmp)
                if hasattr(tmp, 'TotalPages'):
                    pages = int(tmp.TotalPages)
                    page = 2
                    while page <= pages and page < 11:
                        tmp = amazon.searchByTitle(self.title, type='ItemAttributes', product_line='Video', locale=self.locale, page=page)
                        self.result.append(tmp)
                        page = page + 1
            except amazon.AmazonError:
                log.exception('Error retrieving results from amazon.')
            # if all digits then try to find an EAN / UPC
            if self.title.isdigit():
                if len(self.title) == 13:
                    try:
                        tmp = amazon.searchByEAN(self.title, type='ItemAttributes', product_line='Video', locale=self.locale)
                        self.result.append(tmp)
                    except amazon.AmazonError, e:
                        log.exception('Error retrieving results from amazon.')
                elif len(self.title) == 12:
                    try:
                        tmp = amazon.searchByUPC(self.title, type='ItemAttributes', product_line='Video', locale=self.locale)
                        self.result.append(tmp)
                    except amazon.AmazonError, e:
                        log.exception('Error retrieving results from amazon.')
        except IOError:
            log.exception('Error retrieving results from amazon.')
            #self.progress.dialog.hide()
            #gutils.urllib_error(_('Connection error'), self.parent_window)
            #self.suspend()

    def run_get(self):
        self.result = None
        try:
            # get by ASIN
            try:
                self.result = amazon.searchByASIN(self.title, type='Large', locale=self.locale)
            except amazon.AmazonError, e:
                log.exception('Error retrieving results from amazon.')
        except IOError:
            log.exception('Error retrieving results from amazon.')
            #self.progress.dialog.hide()
            #gutils.urllib_error(_('Connection error'), self.parent_window)
            #self.suspend()


#
# Plugin Test
#
# Amazon test data for DE locale only
#
class SearchPluginTest(SearchPlugin):
    #
    # Configuration for automated tests:
    # dict { movie_id -> [ expected result count for original url, expected result count for translated url ] }
    #
    test_configuration = {
        'Rocky Balboa' : [  22,  22 ],
        'Arahan'       : [   9,   9 ],
        'Glück'        : [ 100, 100 ]
    }


class PluginTest:
    #
    # Configuration for automated tests:
    # dict { movie_id -> dict { arribute -> value } }
    #
    # value: * True/False if attribute only should be tested for any value
    #        * or the expected value
    #
    test_configuration = {
        'B000TIQMMI' : {
            'title'               : 'Rocky Balboa',
            'o_title'             : 'Rocky Balboa',
            'director'            : '',
            'plot'                : True,
            'cast'                : 'Sylvester Stallone\n\
Antonio Traver\n\
Burt Young',
            'country'             : 'USA',
            'genre'               : 'Sport, Drama, Drama, Reise & Freizeit',
            'classification'      : 'Freigegeben ab 12 Jahren',
            'studio'              : 'MGM Home Entertainment GmbH (dt.)',
            'o_site'              : False,
            'site'                : 'http://www.amazon.de/dp/B000TIQMMI',
            'trailer'             : False,
            'year'                : 2006,
            'notes'               : 'EAN: 4045167004504',
            'runtime'             : 97,
            'image'               : True,
            'rating'              : 8,
            'barcode'             : '4045167004504'
        },
        'B0009NSASM' : {
            'title'               : 'Ein glückliches Jahr',
            'o_title'             : 'Ein glückliches Jahr',
            'director'            : 'Claude Lelouch',
            'plot'                : False,
            'cast'                : 'Lino Ventura\n\
Françoise Fabian\n\
Charles Gérard',
            'country'             : 'Frankreich, Italien',
            'genre'               : 'Krimi, Thriller, Krimikomödie, General AAS, Drama, Komödie, Krimi',
            'classification'      : 'Freigegeben ab 12 Jahren',
            'studio'              : 'Warner Home Video - DVD',
            'o_site'              : False,
            'site'                : 'http://www.amazon.de/dp/B0009NSASM',
            'trailer'             : False,
            'year'                : 1973,
            'notes'               : 'EAN: 7321921998843',
            'runtime'             : 110,
            'image'               : True,
            'rating'              : 10,
            'barcode'             : '7321921998843'
        },
        'B000BSNOD6' : {
            'title'               : 'Arahan (Vanilla-DVD)',
            'o_title'             : 'Arahan (Vanilla-DVD)',
            'director'            : 'Ryoo Seung-wan',
            'plot'                : False,
            'cast'                : 'Ryu Seung-beom\n\
Yoon So-yi\n\
Ahn Sung-kee',
            'country'             : 'Südkorea',
            'genre'               : 'Actionkomödie, Abenteuer- & Actionkomödie, Fantasykomödie, General AAS, Action, Komödie, Action, Fantasy, Komödie, Korea',
            'classification'      : 'Freigegeben ab 16 Jahren',
            'o_site'              : False,
            'site'                : 'http://www.amazon.de/dp/B000BSNOD6',
            'trailer'             : False,
            'year'                : 2005,
            'notes'               : 'EAN: 4013549871105',
            'runtime'             : 108,
            'image'               : True,
            'rating'              : 8,
            'barcode'             : '4013549871105'
        }
    }



# -*- coding: UTF-8 -*-

__revision__ = '$Id: movie.py 1585 2011-09-12 20:43:18Z mikej06 $'

# Copyright (c) 2005-2009 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging
import os
import string
import sys
import tempfile
import threading
import time
from urllib import *
import gtk
import gutils
import urllib2

log = logging.getLogger("Griffith")


class Movie(object):
    cast = None
    classification = None
    country = None
    director = None
    genre = None
    image = None
    notes = None
    number = None
    o_site = None
    o_title = None
    plot = None
    rating = None
    runtime = None
    site = None
    studio = None
    title = None
    trailer = None
    year = None
    screenplay = None
    cameraman = None
    resolution = None
    barcode = None

    movie_id = None
    debug = False
    locations = None
    engine_author = None
    engine_description = None
    engine_language = None
    engine_name = None
    engine_version = None
    page = None
    url = None
    image_url = None
    encode = 'iso-8859-1'
    fields_to_fetch = []
    progress = None
    useurllib2 = False

    # functions that plugin should implement: {{{
    def initialize(self):
        pass

    def get_barcode(self):
        pass

    def get_cameraman(self):
        pass

    def get_cast(self):
        pass

    def get_classification(self):
        pass

    def get_country(self):
        pass

    def get_director(self):
        pass

    def get_genre(self):
        pass

    def get_image(self):
        pass

    def get_notes(self):
        pass

    def get_o_site(self):
        pass

    def get_o_title(self):
        pass

    def get_plot(self):
        pass

    def get_rating(self):
        pass

    def get_resolution(self):
        pass

    def get_runtime(self):
        pass

    def get_screenplay(self):
        pass

    def get_site(self):
        pass

    def get_studio(self):
        pass

    def get_title(self):
        pass

    def get_trailer(self):
        pass

    def get_year(self):
        pass
    #}}}

    def __getitem__(self, key):
        return getattr(self, key)

    def __setitem__(self, key, value):
        setattr(self, key, value)

    def get_movie(self, parent_window=None):
        try:
            # check for internet connection
            urllib2.urlopen("http://www.griffith.cc")
            #
            # initialize the progress dialog once for the following loading process
            #
            if self.progress is None:
                self.progress = Progress(parent_window)
            self.progress.set_data(parent_window, _("Fetching data"), _("Wait a moment"), True)
            #
            # get the page
            #
            if not self.open_page(parent_window):
                return None
            return True

        except:
            self.progress.hide()
            gutils.error(_("Connection failed."))
            return False

    def open_page(self, parent_window=None, url=None):
        if url is None:
            url_to_fetch = self.url
        else:
            url_to_fetch = url
        if parent_window is not None:
            self.parent_window = parent_window
        self.progress.set_data(parent_window, _("Fetching data"), _("Wait a moment"), False)
        retriever = Retriever(url_to_fetch, self.parent_window, self.progress, useurllib2=self.useurllib2)
        retriever.start()
        while retriever.isAlive():
            self.progress.pulse()
            if self.progress.status:
                retriever.join()
            while gtk.events_pending():
                gtk.main_iteration()
        data = None
        try:
            if retriever.exception is None:
                if retriever.html:
                    ifile = file(retriever.html[0], "rb")
                    try:
                        data = ifile.read()
                    finally:
                        ifile.close()
                    # check for gzip compressed pages before decoding to unicode
                    if len(data) > 2 and data[0:2] == '\037\213':
                        data = gutils.decompress(data)
                    try:
                        # try to decode it strictly
                        if self.encode:
                            data = data.decode(self.encode)
                    except UnicodeDecodeError, exc:
                        # something is wrong, perhaps a wrong character set
                        # or some pages are not as strict as they should be
                        # (like OFDb, mixes utf8 with iso8859-1)
                        # I want to log the error here so that I can find it
                        # but the program should not terminate
                        log.error(exc)
                        data = data.decode(self.encode, 'ignore')
            else:
                gutils.urllib_error(_("Connection error"), self.parent_window)
        except IOError:
            log.exception('')
        if url is None:
            self.page = data
        urlcleanup()
        return data

    def fetch_picture(self):
        if self.image_url:
            tmp_dest = tempfile.mktemp(prefix='poster_', dir=self.locations['temp'])
            self.image = tmp_dest.split('poster_', 1)[1]
            dest = "%s.jpg" % tmp_dest
            try:
                self.progress.set_data(self.parent_window, _("Fetching poster"), _("Wait a moment"), False)
                retriever = Retriever(self.image_url, self.parent_window, self.progress, dest, useurllib2=self.useurllib2)
                retriever.start()
                while retriever.isAlive():
                    self.progress.pulse()
                    if self.progress.status:
                        retriever.join()
                    while gtk.events_pending():
                        gtk.main_iteration()
                urlcleanup()
            except:
                log.exception('')
                self.image = ""
                try:
                    os.remove("%s.jpg" % tmp_dest)
                except:
                    log.info("Can't remove %s file" % tmp_dest)
        else:
            self.image = ""

    def parse_movie(self):
        try:
            fields = list(self.fields_to_fetch)  # make a copy

            self.initialize()

            if 'year' in fields:
                self.get_year()
                self.year = gutils.digits_only(self.year, 2100)
                fields.pop(fields.index('year'))
            if 'runtime' in fields:
                self.get_runtime()
                self.runtime = gutils.digits_only(self.runtime)
                fields.pop(fields.index('runtime'))
            if 'rating' in fields:
                self.get_rating()
                self.rating = gutils.digits_only(self.rating, 10)
                fields.pop(fields.index('rating'))
            if 'cast' in fields:
                self.get_cast()
                self.cast = gutils.clean(self.cast)
                if not isinstance(self.cast, unicode):
                    self.cast = gutils.gdecode(self.cast, self.encode)
                fields.pop(fields.index('cast'))
            if 'plot' in fields:
                self.get_plot()
                self.plot = gutils.clean(self.plot)
                if not isinstance(self.plot, unicode):
                    self.plot = gutils.gdecode(self.plot, self.encode)
                fields.pop(fields.index('plot'))
            if 'notes' in fields:
                self.get_notes()
                self.notes = gutils.clean(self.notes)
                if not isinstance(self.notes, unicode):
                    self.notes = gutils.gdecode(self.notes, self.encode)
                fields.pop(fields.index('notes'))
            if 'image' in fields:
                self.get_image()
                self.fetch_picture()
                fields.pop(fields.index('image'))

            for i in fields:
                getattr(self, "get_%s" % i)()
                self[i] = gutils.clean(self[i])
                if not isinstance(self[i], unicode):
                    self[i] = gutils.gdecode(self[i], self.encode)

            if 'o_title' in self.fields_to_fetch and self.o_title is not None:
                if self.o_title[:4] == u'The ':
                    self.o_title = self.o_title[4:] + u', The'
            if 'title' in self.fields_to_fetch and self.title is not None:
                if self.title[:4] == u'The ':
                    self.title = self.title[4:] + u', The'
        except:
            log.exception('')
        finally:
            # close the progress dialog which was opened in get_movie
            self.progress.hide()


class SearchMovie(object):
    page = None
    number_results = None
    titles = []
    ids = []
    url = None
    encode = 'utf-8'
    original_url_search = None
    translated_url_search = None
    elements = None
    title = None
    remove_accents = True
    progress = None
    useurllib2 = False

    def __init__(self):
        pass

    def search_movies(self, parent_window):
        try:
            #
            # initialize the progress dialog once for the following search process
            #
            if self.progress is None:
                self.progress = Progress(parent_window)
            self.progress.set_data(parent_window, _("Searching"), _("Wait a moment"), True)
            #
            # call the plugin specific search method
            #
            return self.search(parent_window)
        finally:
            # dont forget to hide the progress dialog
            self.progress.hide()

    def open_search(self, parent_window, destination=None):
        self.titles = [""]
        self.ids = [""]
        if self.url.find('%s') > 0:
            self.url = self.url % self.title
            self.url = string.replace(self.url, ' ', '%20')
        else:
            self.url = string.replace(self.url + self.title, ' ', '%20')
        try:
            url = self.url.encode(self.encode)
        except UnicodeEncodeError:
            url = self.url.encode('utf-8')
        self.progress.set_data(parent_window, _("Searching"), _("Wait a moment"), True)
        retriever = Retriever(url, parent_window, self.progress, destination, useurllib2=self.useurllib2)
        retriever.start()
        while retriever.isAlive():
            self.progress.pulse()
            if self.progress.status:
                retriever.join()
            while gtk.events_pending():
                gtk.main_iteration()
        try:
            if retriever.exception is None:
                if destination:
                    # caller gave an explicit destination file
                    # don't care about the content
                    return True
                if retriever.html:
                    ifile = file(retriever.html[0], 'rb')
                    try:
                        self.page = ifile.read()
                    finally:
                        ifile.close()
                    # check for gzip compressed pages before decoding to unicode
                    if len(self.page) > 2 and self.page[0:2] == '\037\213':
                        self.page = gutils.decompress(self.page)
                    self.page = self.page.decode(self.encode, 'replace')
                else:
                    return False
            else:
                self.progress.hide()
                gutils.urllib_error(_("Connection error"), parent_window)
                return False
        except IOError:
            log.exception('')
        finally:
            urlcleanup()
        return True


class Retriever(threading.Thread):

    def __init__(self, URL, parent_window, progress, destination=None, useurllib2=False):
        self.URL = URL
        self.html = None
        self.exception = None
        self.destination = destination
        self.useurllib2 = useurllib2
        self.progress = progress
        self._stopevent = threading.Event()
        self._sleepperiod = 1.0
        threading.Thread.__init__(self, name="Retriever")

    def run(self):
        try:
            if self.useurllib2:
                self.html = urlretrieve2(self.URL, self.destination, self.hook)
            else:
                self.html = urlretrieve(self.URL, self.destination, self.hook)
            if self.progress.status:
                self.html = []
        except Exception, e:
            log.exception('')
            self.exception = e

    def hook(self, count, blockSize, totalSize):
        if totalSize == -1:
            pass
        else:
            try:
                downloaded_percentage = min((count * blockSize * 100) / totalSize, 100)
            except:
                downloaded_percentage = 100
            if count != 0:
                downloaded_kbyte = int(count * blockSize / 1024.0)
                filesize_kbyte = int(totalSize / 1024.0)

#
# use own derived URLopener class because we need to set a correct User-Agent
# string for some web sites. The default is 'Python-urllib/<version>'
# which not all sites accepting. (zelluloid.de for example)
#
_uaurlopener = None
_tempfilecleanup = None


def urlretrieve(url, filename=None, reporthook=None, data=None):
    global _uaurlopener
    if not _uaurlopener:
        _uaurlopener = UAFancyURLopener()
    return _uaurlopener.retrieve(url, filename, reporthook, data)


def urlretrieve2(url, filename=None, reporthook=None, data=None):
    global _tempfilecleanup
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.9.1.7) Gecko/20091221 Firefox/3.5.7',
        'Cache-Control': 'no-cache',
        'Pragma': 'no-cache',
        'Accept-Encoding': 'gzip'}
    req = urllib2.Request(url, data, headers)
    response = urllib2.urlopen(req)
    if not filename:
        import tempfile
        (fd, filename) = tempfile.mkstemp()
        if not _tempfilecleanup:
            _tempfilecleanup = TempFileCleanup()
        _tempfilecleanup._tempfiles.append(filename)
        tfp = os.fdopen(fd, 'wb')
    else:
        tfp = file(filename, 'wb')
    while 1:
        block = response.read(4096)
        if block == "":
            break
        tfp.write(block)
    tfp.close()
    return [filename, response.info()]


class UAFancyURLopener(FancyURLopener):
    # use Firefox 3.0 User-Agent string from Windows XP
    version = 'Mozilla/5.0 (Windows; U; Windows NT 6.0) Gecko/2008052906 Firefox/3.0'

    def __init__(self, *args, **kwargs):
        FancyURLopener.__init__(self, *args, **kwargs)
        # additional HTTP headers to work around the html file compression which
        # is used by UMTS connections. compression breaks the movie import plugins sometimes
        self.addheaders.append(('Cache-Control', 'no-cache'))
        self.addheaders.append(('Pragma', 'no-cache'))

    def open(self, fullurl, data=None):
        # prevent blocking calls which doesn't come back
        import socket
        socket.setdefaulttimeout(20)
        return FancyURLopener.open(self, fullurl, data)


class TempFileCleanup:
    _tempfiles = []

    def __init__(self):
        self.__unlink = os.unlink

    def __del__(self):
        self.cleanup()

    def cleanup(self):
        if self._tempfiles:
            for file in self._tempfiles:
                try:
                    self.__unlink(file)
                except OSError:
                    pass
            del self._tempfiles[:]


class Progress:

    def __init__(self, window, title='', message=''):
        self.status = False
        self.dialog = gtk.Dialog(title, window, gtk.DIALOG_MODAL, ())
        self.dialog.set_urgency_hint(False)
        self.label = gtk.Label()
        self.label.set_markup(message)
        self.dialog.vbox.pack_start(self.label)
        self.progress = gtk.ProgressBar()
        self.progress.set_pulse_step(0.01)
        self.dialog.vbox.pack_start(self.progress, False, False)
        self.button = gtk.Button(_("Cancel"), gtk.STOCK_CANCEL)
        self.button.connect("clicked", self.callback)
        self.dialog.vbox.pack_start(self.button, False, False)
        self.dialog.show_all()

    def callback(self, widget):
        self.dialog.hide()
        self.status = True

    def pulse(self):
        self.progress.pulse()
        time.sleep(0.01)

    def close(self):
        self.dialog.destroy()

    def hide(self):
        self.dialog.hide()

    def set_data(self, parent_window, title, message, showit):
        #self.dialog.set_parent(parent_window)
        self.dialog.set_title(title)
        self.label.set_markup(message)
        if showit is True:
            self.dialog.show()
# vim: fdm=marker



# -*- coding: utf-8 -*-

__revision__ = '$Id: PluginMovieFDb.py 1385 2010-01-06 19:47:44Z kura666 $'

# Copyright (c) 2006-2007 Piotr Ozarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string
import re

plugin_name        = 'FDb'
plugin_description    = 'Internetowa baza filmowa'
plugin_url        = 'fdb.pl'
plugin_language        = _('Polish')
plugin_author        = 'Piotr Ożarowski, Bartosz Kurczewski'
plugin_author_email    = '<bartosz.kurczewski@gmail.com>'
plugin_version        = '1.16'

class Plugin(movie.Movie):
    TRAILER_PATTERN = re.compile('/film/.*/zwiastuny/odtwarzaj/[0-9]*')

    def __init__(self, movie_id):
        from md5 import md5
        self.movie_id = md5(movie_id).hexdigest()
        self.encode   = 'utf-8'
        if string.find(movie_id, 'http://') != -1:
            self.url = str(movie_id)
        else:
            self.url = "http://fdb.pl/%s" % str(movie_id)

    def get_image(self):
        self.image_url = gutils.trim(self.page, 'class="gfx-poster"', '/></a>')
        self.image_url = gutils.trim(self.image_url,'src="','"')
        if self.image_url.endswith('http://fdb.pl/images/fdb2/add_poster.png?1214306282'):
            self.image_url = ''

    def get_o_title(self):
        self.o_title = gutils.trim(self.page, '<h2 class="after-title">', '</h2>')
        self.o_title = gutils.strip_tags(self.o_title)
        self.o_title = string.strip(self.o_title)
        if self.o_title == '':
            self.o_title = self.get_title(True)

    def get_title(self, extra=False):
        data = gutils.trim(self.page,'<h1 id="movie-title">', '</h1>')
        tmp = string.find(data, '(')
        if tmp != -1:
            data = data[:tmp]
        if extra is False:
            self.title = data
        else:
            return data

    def get_director(self):
        self.director = ''
        elements = gutils.trim(self.page,'>Reżyseria: </h4>','</div>')
        elements = string.split(elements, '</li>')
        if elements[0] != '':
            for element in elements:
                element = gutils.trim(element, '>', '</a')
                if element != '':
                    self.director += ', ' + element
            self.director = string.replace(self.director[2:], ', &nbsp;&nbsp;&nbsp;(więcej)', '')

    def get_screenplay(self):
        self.screenplay = ''
        elements = gutils.trim(self.page,'>Scenariusz: </h4>','</div>')
        elements = string.split(elements, '</li>')
        if elements[0] != '':
            for element in elements:
                element = gutils.trim(element, '>', '</a')
                if element != '':
                    self.screenplay += ', ' + element
            self.screenplay = string.replace(self.screenplay[2:], ', &nbsp;&nbsp;&nbsp;(więcej)', '')


    def get_plot(self):
        self.plot = gutils.trim(self.page,'Opis filmu','</div>')
        self.plot = gutils.trim(self.plot,'<p>',"\n\n")

    def get_year(self):
        self.year = gutils.trim(self.page,'<small>  (', ')</small>')

    def get_runtime(self):
        self.runtime = gutils.trim(self.page,'Czas trwania: </h4>','</div>')
        self.runtime = gutils.trim(self.runtime,'<li>',' minut')

    def get_genre(self):
        self.genre = gutils.trim(self.page,'Gatunek: </h4>','</div>')
        self.genre = gutils.trim(self.genre,'<li>','</li>')
        self.genre = string.replace(self.genre, ' / ', ' | ')

    def get_cast(self):
        self.cast = gutils.trim(self.page,'<div class="cast">',"</div>")
        tmpcast = gutils.trim(self.cast,"<table>\n",'</table>')
        if tmpcast == '':
            tmpcast = tmpcast = gutils.trim(self.cast,"<table>\n",'<div class="line">')
        self.cast = tmpcast   
        if self.cast != '':
            self.cast = gutils.strip_tags(self.cast)
            self.cast = self.cast.replace("      ","")
            self.cast = self.cast.replace("    ","")
            self.cast = self.cast.replace("\n...\n\n  ",_(' as '))
            self.cast = self.cast.replace("\n\n\n\n\n","")

    def get_classification(self):
        self.classification = gutils.trim(self.page,"Od lat: </h4>","</div>")
        self.classification = gutils.trim(self.classification,'<li>','</li>')

    def get_studio(self):
        self.studio = ''

    def get_o_site(self):
        self.o_site = ''

    def get_site(self):
        self.site = self.url

    def get_trailer(self):
        trailer_url = self.TRAILER_PATTERN.findall(self.page)
        if trailer_url:
            self.trailer = 'http://fdb.pl' + trailer_url[0]

    def get_country(self):
        self.country = gutils.trim(self.page,'Kraj produkcji: </h4>','</div>')
        self.country = gutils.trim(self.country,'<li>','</li>')

    def get_rating(self):
        self.rating = gutils.trim(self.page, 'class="vote"','/10</strong>')
        self.rating = gutils.after(self.rating, '<strong>')
        if self.rating:
            self.rating = str(float(gutils.clean(self.rating)))

class SearchPlugin(movie.SearchMovie):
    def __init__(self):
        self.encode = 'utf-8'
        self.original_url_search    = 'http://fdb.pl/szukaj/movies?query='
        self.translated_url_search    = 'http://fdb.pl/szukaj/movies?query='

    def search(self,parent_window):
        if not self.open_search(parent_window):
            return None
        tmp = string.find(self.page,'<h1>Wyniki wyszukiwania dla')
        if tmp == -1:        # already a movie page
            self.page = ''
        else:            # multiple matches
            self.page = gutils.before(self.page[tmp:],'>Mapa strony</h3>');
        return self.page

    def get_searches(self):
        if self.page == '':    # movie page already
            self.number_results = 1
            self.ids.append(self.url)
            self.titles.append(self.title)
        else:            # multiple matches
            elements = string.split(self.page,'<div class="content">')
            if len(elements)>1:
                for element in elements:
                    tmpId = gutils.trim(element, '<a href="', '"')
                    if tmpId.startswith('/szukaj'):
                        continue
                    self.ids.append(tmpId)
                    element = gutils.strip_tags(
                        gutils.trim(element, '">', '</div>'))
                    element = element.replace("\n", '')
                    element = element.replace('   ', '')
                    element = element.replace('aka ', ' aka ')
		    element = element.replace('{{rate}}', ' ')
                    element = element.replace(' - Oryginalny', '')
                    self.titles.append(element)
            else:
                self.number_results = 0



# -*- coding: UTF-8 -*-

__revision__ = '$Id: ge_amazon.py 1568 2011-08-10 20:08:12Z mikej06 $'

# Copyright © 2009 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published byp
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging
import os
import tempfile
from urllib import urlcleanup, FancyURLopener, urlretrieve

import gtk
from PIL import Image

import gutils
from plugins.extensions import GriffithExtensionBase as Base
from widgets import populate_results_window
from edit import update_image

import amazon

log = logging.getLogger('Griffith')


class GriffithExtension(Base):
    name = 'Amazon'
    description = _('Fetch from Amazon')
    author = 'Piotr Ożarowski'
    email = 'piotr@griffith.cc'
    version = 1
    api = 1
    enabled = False

    preferences = {'locale': {'name': _('Select source'),
                              'default': u'US',
                              'type': (u'US', u'UK', u'DE', u'CA', u'FR', u'JP')},
                   'accesskey': {'name': _('Access Key ID'),
                                 'hint': _('Get your Access Key ID from ') + u'https://affiliate-program.amazon.com/gp/flex/advertising/api/sign-in.html',
                                 'default': u'',
                                 'type': unicode},
                   'secretkey': {'name': _('Secret Key'),
                                 'hint': _('Get your Secret Key from ') + u'https://affiliate-program.amazon.com/gp/flex/advertising/api/sign-in.html',
                                 'default': u'',
                                 'type': unicode}}
    toolbar_icon = 'ge_amazon.png'

    def toolbar_icon_clicked(self, widget, movie):
        log.info('fetching poster from Amazon...')
        self.movie = movie

        locale = self.get_config_value('locale', 'US').lower()
        accesskey = self.get_config_value('accesskey')
        secretkey = self.get_config_value('secretkey')

        if not accesskey or not secretkey:
            gutils.error(_('Please configure your Amazon Access Key ID and Secret Key correctly in the preferences dialog.'))
            return False

        if movie is None:
            gutils.error(_('You have no movies in your database'), self.widgets['window'])
            return False

        keyword = movie.o_title
        if locale == 'de':
            keyword = movie.title

        try:
            amazon.setLicense(accesskey, secretkey)
            result = amazon.searchByTitle(keyword, type='Large', product_line='DVD', locale=locale)
            if hasattr(result, 'TotalPages'):
                # get next result pages
                pages = int(result.TotalPages)
                page = 2
                while page <= pages and page < 11:
                    tmp = amazon.searchByTitle(keyword, type='Large', product_line='DVD', locale=locale, page=page)
                    result.Item.extend(tmp.Item)
                    page = page + 1
            if not hasattr(result, 'Item') or not len(result.Item):
                # fallback if nothing is found by title
                result = amazon.searchByKeyword(keyword, type='Large', product_line='DVD', locale=locale)
                if hasattr(result, 'TotalPages'):
                    # get next result pages
                    pages = int(result.TotalPages)
                    page = 2
                    while page <= pages and page < 11:
                        tmp = amazon.searchByKeyword(keyword, type='Large', product_line='DVD', locale=locale, page=page)
                        result.Item.extend(tmp.Item)
                        page = page + 1
            self._result = result
            log.info("... %s posters found" % result.TotalResults)
        except:
            log.exception('')
            gutils.warning(_('No posters found for this movie.'))
            return

        if not hasattr(result, 'Item') or not len(result.Item):
            gutils.warning(_('No posters found for this movie.'))
            return

        if len(result.Item) == 1:
            o_title = self.widgets['movie']['o_title'].get_text().decode('utf-8')
            if o_title == result.Item[0].ItemAttributes.Title or keyword == result.Item[0].ItemAttributes.Title:
                self.get_poster(self.app, result.Item[0])
                return

        # populate results window
        items = {}
        for i, item in enumerate(result.Item):
            if hasattr(item, 'LargeImage') and len(item.LargeImage.URL):
                title = item.ItemAttributes.Title
                if hasattr(item.ItemAttributes, 'ProductGroup'):
                    title = title + u' - ' + item.ItemAttributes.ProductGroup
                elif hasattr(item.ItemAttributes, 'Binding'):
                    title = title + u' - ' + item.ItemAttributes.Binding
                if hasattr(item.ItemAttributes, 'ReleaseDate'):
                    title = title + u' - ' + item.ItemAttributes.ReleaseDate[:4]
                elif hasattr(item.ItemAttributes, 'TheatricalReleaseDate'):
                    item.ItemAttributes.TheatricalReleaseDate[:4]
                if hasattr(item.ItemAttributes, 'Studio'):
                    title = title + u' - ' + item.ItemAttributes.Studio
                items[i] = title

        populate_results_window(self.app.treemodel_results, items)
        self.widgets['add']['b_get_from_web'].set_sensitive(False) # disable movie plugins (result window is shared)
        self.app._resultswin_process = self.on_result_selected # use this signal (will be reverted when window is closed)
        self.widgets['results']['window'].show()
        self.widgets['results']['window'].set_keep_above(True)

    def on_result_selected(self, key):
        key = int(key)
        if self._result.Item[key].LargeImage.URL:
            poster_file = self.get_poster(self._result.Item[key])
            if poster_file:
                update_image(self.app, self.movie.number, poster_file)
                try:
                    os.remove(poster_file)
                except:
                    log.error('cannot remove %s', poster_file)
        self.widgets['results']['window'].hide()
        self.widgets['add']['b_get_from_web'].set_sensitive(True)

    def get_poster(self, item):
        """Returns file path to the new poster"""

        from movie import Progress, Retriever

        file_to_copy = tempfile.mktemp(suffix=self.widgets['movie']['number'].get_text(), \
            dir=self.locations['temp'])
        file_to_copy += ".jpg"
        canceled = False
        try:
            progress = Progress(self.widgets['window'], _("Fetching poster"), _("Wait a moment"))
            retriever = Retriever(item.LargeImage.URL, self.widgets['window'], progress, file_to_copy)
            retriever.start()
            while retriever.isAlive():
                progress.pulse()
                if progress.status:
                    canceled = True
                while gtk.events_pending():
                    gtk.main_iteration()
            progress.close()
            urlcleanup()
        except:
            canceled = True
            gutils.warning(_("Sorry. A connection error has occurred."))
            try:
                os.remove(file_to_copy)
            except:
                log.error("no permission for %s" % file_to_copy)

        if not canceled:
            if os.path.isfile(file_to_copy):
                im = None
                try:
                    im = Image.open(file_to_copy)
                except IOError:
                    log.warn("failed to identify %s" % file_to_copy)

                if im and im.size == (1, 1):
                    url = FancyURLopener().open("http://www.amazon.com/gp/product/images/%s" % item.ASIN).read()
                    if url.find('no-img-sm._V47056216_.gif') > 0:
                        log.warn('No image available')
                        gutils.warning(_("Sorry. This movie is listed but has no poster available at Amazon.com."))
                        return False
                    url = gutils.after(url, 'id="imageViewerDiv"><img src="')
                    url = gutils.before(url, '" id="prodImage"')
                    urlretrieve(url, file_to_copy)
                    try:
                        im = Image.open(file_to_copy)
                    except IOError:
                        log.warn("failed to identify %s", file_to_copy)

                if not im:
                    # something wrong with the image, give some feedback to the user
                    log.warn('No image available')
                    gutils.warning(_("Sorry. This movie is listed but has no poster available at Amazon.com."))
                    return False

                if im.mode != 'RGB': # convert GIFs
                    im = im.convert('RGB')
                    im.save(file_to_copy, 'JPEG')
                # set to None because the file is locked otherwise (os.remove throws an exception)
                im = None

                handler = self.widgets['big_poster'].set_from_file(file_to_copy)

                self.widgets['poster_window'].show()
                self.widgets['poster_window'].move(0, 0)
                if gutils.question(_("Do you want to use this poster instead?"), self.widgets['window']):
                    return file_to_copy
                else:
                    log.info("Reverting to previous poster and deleting new one from disk.")
                    try:
                        os.remove(file_to_copy)
                    except:
                        log.error('cannot remove %s', file_to_copy)

                self.widgets['poster_window'].hide()
            else:
                gutils.warning(_("Sorry. This movie is listed but has no poster available at Amazon.com."))
        else:
            # cleanup temporary files after canceling the download
            if os.path.isfile(file_to_copy):
                try:
                    os.remove(file_to_copy)
                except:
                    log.error('cannot remove %s', file_to_copy)



# -*- coding: iso-8859-15 -*-

__revision__ = '$Id: PluginMovieCulturalia.py 389 2006-07-29 18:43:35Z piotrek $'

# Copyright (c) 2006 Pedro D. Sánchez
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string

plugin_name         = 'Culturalia'
plugin_description  = 'Base de Datos de Peliculas'
plugin_url          = 'www.culturalianet.com'
plugin_language     = _('Spanish')
plugin_author       = 'Pedro D. Sánchez'
plugin_author_email = '<pedrodav@gmail.com>'
plugin_version      = '0.3'

class Plugin(movie.Movie):
    def __init__(self, id):
        self.encode   = 'iso-8859-1'
        self.movie_id = id
        self.url      = "http://www.culturalianet.com/art/ver.php?art=%s" % str(self.movie_id)

    def get_image(self):
        tmp = string.find(self.page, "<font class = 'titulo2'>")
        if tmp == -1:
            self.image_url = ''
        else:
            self.image_url = 'http://www.culturalianet.com/art/'+gutils.trim(self.page[tmp:], '<img src = "', '"')

    def get_o_title(self):
        self.o_title = gutils.trim(self.page, "<font class = 'titulo2'>", '</i>')
        self.o_title = gutils.after(self.o_title, '<i>')

    def get_title(self):
        self.title = gutils.trim(self.page, "<font class = 'titulo2'>", '. (')

    def get_director(self):
        self.director = gutils.trim(self.page,"<font class = 'titulo3'>Director:", '<br><br>')

    def get_plot(self):
        self.plot = gutils.trim(self.page, '<b>Sinopsis:</b><br>', '<br><br>')

    def get_year(self):
        self.year = gutils.trim(self.page, "<font class = 'titulo2'>", ')</font>')
        self.year = gutils.after(self.year, '. (')

    def get_runtime(self):
        self.runtime = gutils.trim(self.page, u"<font class = 'titulo3'>Duración:</font> ", u' minutos')
        if self.runtime == '':
            self.runtime = gutils.trim(self.page, "<font class = 'titulo3'>Duraci&oacute;n:</font> ", ' minutos')

    def get_genre(self):
        self.genre = gutils.trim(self.page, u"<font class = 'titulo3'>Género:</font><br>", u'<br><br>')

    def get_cast(self):
        self.cast = ''
        self.cast = gutils.trim(self.page, "<font class = 'titulo3'>Actores:</font><br>", '<br><br>')
        self.cast = string.replace(self.cast, '<br>', "\n")
        self.cast = string.strip(gutils.strip_tags(self.cast))

    def get_classification(self):
        self.classification = gutils.trim(self.page, u"<font class = 'titulo3'>Calificación moral:</font> ", u'<br>')
        if self.classification == '':
            self.classification = gutils.trim(self.page, "<font class = 'titulo3'>Calificaci&oacute;n moral:</font> ", '<br>')

    def get_studio(self):
        self.studio = ''

    def get_o_site(self):
        self.o_site = ''

    def get_site(self):
        self.site = "http://www.culturalianet.com/art/ver.php?art=%s" % str(self.movie_id)

    def get_trailer(self):
        self.trailer = ''

    def get_country(self):
        self.country = gutils.trim(self.page, "<font class = 'titulo3'>Nacionalidad:</font><br>", '<br>')

    def get_rating(self):
        self.rating = gutils.trim(self.page, u"Puntuación: <font class = 'titulo3'>", u'</font>')
        if self.rating == '':
            self.rating = gutils.trim(self.page, "Puntuaci&oacute;n: <font class = 'titulo3'>", '</font>')
        if self.rating:
            self.rating = str(float(gutils.clean(self.rating)))

    def get_screenplay(self):
        self.screenplay = gutils.trim(self.page,u'>Guión:', u'<br><br>')

    def get_cameraman(self):
        self.cameraman = gutils.trim(self.page,u'>Fotografía:', u'<br><br>')

class SearchPlugin(movie.SearchMovie):

    def __init__(self):
        self.original_url_search   = 'http://www.culturalianet.com/bus/resu.php?donde=1&texto='
        self.translated_url_search = 'http://www.culturalianet.com/bus/resu.php?donde=1&texto='
        self.encode                = 'iso-8859-1'

    def search(self,parent_window):
        if not self.open_search(parent_window):
            return None
        self.sub_search()
        return self.page

    def sub_search(self):
        self.page = gutils.trim(self.page, '<b>ARTICULOS ENCONTRADOS:</b>', '</table>')
        self.page = gutils.after(self.page, '</td></tr><tr><td><b>')
        #self.page = self.page.decode('iso-8859-15')

    def get_searches(self):
        elements = string.split(self.page, '<td><b>')

        if (elements[0]<>''):
            for element in elements:
                self.ids.append(gutils.trim(element, 'ver.php?art=',"'"))
                self.titles.append(gutils.strip_tags(gutils.convert_entities(gutils.trim(element, "target='_top'>", '</a>'))))

#
# Plugin Test
#
class SearchPluginTest(SearchPlugin):
    #
    # Configuration for automated tests:
    # dict { movie_id -> [ expected result count for original url, expected result count for translated url ] }
    #
    test_configuration = {
        'Rocky Balboa' : [ 11, 11 ],
    }

class PluginTest:
    #
    # Configuration for automated tests:
    # dict { movie_id -> dict { arribute -> value } }
    #
    # value: * True/False if attribute only should be tested for any value
    #        * or the expected value
    #
    test_configuration = {
        '27039' : { 
            'title'               : 'Rocky Balboa',
            'o_title'             : 'Rocky Balboa',
            'director'            : 'Sylvester Stallone',
            'plot'                : True,
            'cast'                : 'Sylvester Stallone\n\
Burt Young\n\
Milo Ventimiglia\n\
Geraldine Hughes\n\
James Francis Kelly III\n\
Tony Burton\n\
A.J. Benza\n\
Talia Shire\n\
Henry G. Sanders\n\
Antonio Tarver\n\
Pedro Lovell\n\
Ana Gerena\n\
Angela Boyd\n\
Louis Giansante\n\
Maureen Schilling',
            'country'             : 'USA',
            'genre'               : 'Drama / Deportes',
            'classification'      : u'No recom. menores de 7 años',
            'studio'              : False,
            'o_site'              : False,
            'site'                : 'http://www.culturalianet.com/art/ver.php?art=27039',
            'trailer'             : False,
            'year'                : 2006,
            'notes'               : False,
            'runtime'             : 99,
            'image'               : True,
            'rating'              : False,
            'cameraman'           : 'J. Clark Mathis',
            'screenplay'          : 'Sylvester Stallone',
            'barcode'             : False
        },
    }



# -*- coding: UTF-8 -*-

__revision__ = '$Id: PluginMovieIMDB.py 176 2006-02-01 12:07:26Z iznogoud $'

# Copyright (c) 2005-2011 Vasco Nunes, Piotr Ozarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, orprint
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import movie
try:
    import simplejson as json
except:
    import json
import logging
log = logging.getLogger("Griffith")

plugin_name         = "Allocine"
plugin_description  = "Internet Movie Database"
plugin_url          = "www.allocine.fr"
plugin_language     = _("French")
plugin_author       = "Pierre-Luc Levy, Michael Jahn (JSON api)"
plugin_author_email = ""
plugin_version      = "1.0"


class Plugin(movie.Movie):
    def __init__(self, id):
        self.movie_id = id
        self.url      = "http://api.allocine.fr/xml/movie?partner=3&json=1&profile=large&code=%s" % str(self.movie_id)
        self.encode   = 'utf-8'

    def initialize(self):
        self.movie = json.JSONDecoder().decode(self.page)['movie']

    def get_image(self):
        self.image_url = ''
        if 'poster' in self.movie:
            self.image_url = self.movie['poster']['href']

    def get_o_title(self):
        self.o_title = ''
        if 'originalTitle' in self.movie:
            self.o_title = self.movie['originalTitle']
        elif 'title' in self.movie:
            self.o_title = self.movie['title']

    def get_title(self):
        self.title = ''
        if 'title' in self.movie:
            self.title = self.movie['title']
        elif 'originalTitle' in self.movie:
            self.title = self.movie['originalTitle']

    def get_director(self):
        self.director = self.buildfromcast(8002)

    def get_plot(self):
        self.plot = ''
        if 'synopsis' in self.movie:
            self.plot = self.movie['synopsis']

    def get_year(self):
        self.year = ''
        if 'productionYear' in self.movie:
            self.year = self.movie['productionYear']

    def get_runtime(self):
        self.runtime = ''
        if 'runtime' in self.movie:
            self.runtime = self.movie['runtime'] / 60

    def get_genre(self):
        self.genre = ''
        if 'genre' in self.movie:
            for genre in self.movie['genre']:
                self.genre = self.genre + genre['$'] + ', '
        if self.genre:
            self.genre = self.genre[:-2]

    def get_cast(self):
        self.cast = ''
        if 'castMember' in self.movie:
            for cast in self.movie['castMember']:
                if 'activity' in cast:
                    activity = cast['activity']
                    if 'code' in activity:
                        if activity['code'] == 8001:
                            if 'role' in cast:
                                self.cast = self.cast + cast['person']['$'] + _(' as ') + cast['role'] + '\n'
                            else:
                                self.cast = self.cast + cast['person']['$'] + '\n'

    def get_classification(self):
        self.classification = ""
        if 'movieCertificate' in self.movie:
            self.classification = self.movie['movieCertificate']['$']

    def get_studio(self):
        self.studio = ""

    def get_o_site(self):
        self.o_site = ""

    def get_site(self):
        self.site = "http://www.allocine.fr/film/fichefilm_gen_cfilm=%s.html" % self.movie_id
        if 'link' in self.movie:
            for link in self.movie['link']:
                if link['rel'] == 'aco:more':
                    self.site = link['href']

    def get_trailer(self):
        self.trailer = "http://www.allocine.fr/film/video_gen_cfilm=%s.html" % self.movie_id
        if 'trailer' in self.movie:
            self.trailer = self.movie['trailer']['href']

    def get_country(self):
        self.country = ''
        if 'nationality' in self.movie:
            for country in self.movie['nationality']:
                self.country = self.country + country['$'] + ', '
        if self.country:
            self.country = self.country[:-2]

    def get_rating(self):
        self.rating = 0
        if 'statistics' in self.movie:
            statistics = self.movie['statistics']
            if 'pressRating' in statistics:
                self.rating = statistics['pressRating'] * 2
            elif 'userRating' in statistics:
                self.rating = statistics['userRating'] * 2

    def get_screenplay(self):
        self.screenplay = self.buildfromcast(8004)

    def get_cameraman(self):
        self.cameraman = self.buildfromcast(8037)

    def buildfromcast(self, code):
        result = ''
        if 'castMember' in self.movie:
            for cast in self.movie['castMember']:
                if 'activity' in cast:
                    activity = cast['activity']
                    if 'code' in activity:
                        if activity['code'] == code:
                            result = result + cast['person']['$'] + ', '
        if result:
            result = result[:-2]
        return result


class SearchPlugin(movie.SearchMovie):

    def __init__(self):
        self.original_url_search   = "http://api.allocine.fr/xml/search?partner=3&json=1&count=100&profile=small&q="
        self.translated_url_search = "http://api.allocine.fr/xml/search?partner=3&json=1&count=100&profile=small&q="
        self.encode                = 'utf-8'
        self.remove_accents        = True

    def search(self, parent_window):
        if not self.open_search(parent_window):
            return None
        return self.page

    def get_searches(self):
        result = json.JSONDecoder().decode(self.page)
        try:
            movies = result['feed']['movie']
            for movie in movies:
                try:
                    title = ''
                    year = ''
                    if 'title' in movie:
                        title = movie['title']
                    elif 'originalTitle' in movie:
                        title = movie['originalTitle']
                    if 'productionYear' in movie:
                        year = '(%s)' % movie['productionYear']
                    self.titles.append('%s %s' % (title, year))
                    self.ids.append(movie['code'])
                except:
                    log.exception('')
        except:
            log.exception('')


#
# Plugin Test
#


class SearchPluginTest(SearchPlugin):
    #
    # Configuration for automated tests:
    # dict { movie_id -> [ expected result count for original url, expected result count for translated url ] }
    #
    test_configuration = {
        'Le Prix à payer' : [ 4, 4 ],
    }


class PluginTest:
    #
    # Configuration for automated tests:
    # dict { movie_id -> dict { arribute -> value } }
    #
    # value: * True/False if attribute only should be tested for any value
    #        * or the expected value
    #
    test_configuration = {
        '110585' : {
            'title'               : u'Le Prix à payer',
            'o_title'             : u'Le Prix à payer',
            'director'            : u'Alexandra Leclère',
            'plot'                : True,
            'cast'                : u'Christian Clavier' + _(' as ') + 'Jean-Pierre Ménard\n\
Nathalie Baye' + _(' as ') + 'Odile Ménard\n\
Gérard Lanvin' + _(' as ') + 'Richard\n\
Géraldine Pailhas' + _(' as ') + 'Caroline\n\
Patrick Chesnais' + _(' as ') + 'Grégoire l\'amant\n\
Anaïs Demoustier' + _(' as ') + 'Justine\n\
Maud Buquet' + _(' as ') + 'la prostituée\n\
Francis Leplay' + _(' as ') + 'l\'agent immobilier',
            'country'             : u'France',
            'genre'               : u'Comédie',
            'classification'      : False,
            'studio'              : False,
            'o_site'              : False,
            'site'                : 'http://www.allocine.fr/film/fichefilm_gen_cfilm=110585.html',
            'trailer'             : 'http://www.allocine.fr/blogvision/18726250',
            'year'                : 2007,
            'notes'               : False,
            'runtime'             : 95,
            'image'               : True,
            'rating'              : 4,
            'cameraman'           : u'Jean-François Robin',
            'screenplay'          : u'Alexandra Leclère',
            'barcode'             : False
        },
        '309' : {
            'title'               : u'Terminator',
            'o_title'             : u'Terminator, The',
            'director'            : u'James Cameron',
            'plot'                : True,
            'cast'                : u'Arnold Schwarzenegger' + _(' as ') + 'le Terminator\n\
Michael Biehn' + _(' as ') + 'Kyle Reese\n\
Linda Hamilton' + _(' as ') + 'Sarah Connor\n\
Lance Henriksen' + _(' as ') + 'l\'inspecteur Vukovich\n\
Paul Winfield' + _(' as ') + 'le lieutenant Ed Traxler\n\
Bess Motta' + _(' as ') + 'Ginger Ventura\n\
Rick Rossovich' + _(' as ') + 'Matt Buchanan\n\
Earl Boen' + _(' as ') + 'le Dr Peter Silberman\n\
Dick Miller' + _(' as ') + 'le marchand d\'armes\n\
Shawn Schepps' + _(' as ') + 'Nancy\n\
Bill Paxton' + _(' as ') + 'le chef des punks\n\
Brian Thompson' + _(' as ') + 'un punk\n\
Marianne Muellerleile' + _(' as ') + 'la \'mauvaise\' Sarah Connor\n\
Franco Columbu' + _(' as ') + 'le Terminator infiltrant le bunker dans le futur\n\
Ken Fritz' + _(' as ') + 'Policeman\n\
Stan Yale' + _(' as ') + 'Derelict in Alley\n\
Brad Rearden' + _(' as ') + 'Punk\n\
Joe Farago' + _(' as ') + 'TV Anchorman\n\
Anthony Trujillo' + _(' as ') + 'Mexican Boy (close-ups)\n\
Harriet Medin' + _(' as ') + 'Customer\n\
Hugh Farrington' + _(' as ') + 'Customer\n\
Philip Gordon' + _(' as ') + 'Mexican Boy (long shots)\n\
Patrick Pinney' + _(' as ') + 'Bar Customer\n\
Wayne Stone' + _(' as ') + 'Tanker Driver\n\
Norman Friedman' + _(' as ') + 'Cleaning Man at Flophouse\n\
Hettie Lynne Hurtes' + _(' as ') + 'TV Anchorwoman\n\
Al Kahn' + _(' as ') + 'Customer\n\
Bill W. Richmond' + _(' as ') + 'Barman\n\
Bruce M. Kerner' + _(' as ') + 'Desk Sergeant\n\
David Pierce' + _(' as ') + 'Tanker Partner\n\
Barbara Powers' + _(' as ') + 'Ticket Taker at Club Technoir\n\
Ed Dogans' + _(' as ') + 'Cop in Alley\n\
Tony Mirelez' + _(' as ') + 'Gas Station Attendant\n\
Webster Williams' + _(' as ') + 'Reporter\n\
John E. Bristol' + _(' as ') + 'Biker at Phone Booth\n\
Gregory Robbins' + _(' as ') + 'Tiki Motel Customer\n\
Chino \'Fats\' Williams' + _(' as ') + 'Truck Driver',
            'country'             : 'U.S.A.',
            'genre'               : 'Science fiction, Thriller',
            'classification'      : 'Interdit aux moins de 12 ans',
            'studio'              : False,
            'o_site'              : False,
            'site'                : 'http://www.allocine.fr/film/fichefilm_gen_cfilm=309.html',
            'trailer'             : 'http://www.allocine.fr/blogvision/18895020',
            'year'                : 1984,
            'notes'               : False,
            'runtime'             : 108,
            'image'               : True,
            'rating'              : 8,
            'cameraman'           : u'Adam Greenberg',
            'screenplay'          : u'James Cameron, Gale Anne Hurd, Harlan Ellison, William Wisher',
            'barcode'             : False
        },
    }



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2010
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string
import re

plugin_name         = "Clubedevideo"
plugin_description  = "www.clubedevideo.com"
plugin_url          = "www.clubedevideo.com"
plugin_language     = _("Portuguese")
plugin_author       = "Michael Jahn"
plugin_author_email = "mike@griffith.cc"
plugin_version      = "1.0"


class Plugin(movie.Movie):

    def __init__(self, id):
        self.encode = 'iso-8859-1'
        self.movie_id = id
        self.url = 'http://www.clubedevideo.com/mods/ficha_filme_if.cdv?numero_filme=%s' % self.movie_id

    def get_image(self):
        self.image_url = 'http://www.clubedevideo.com/capas/%s.jpg' % self.movie_id

    def get_o_title(self):
        self.o_title = gutils.after(gutils.trim(self.page, "class='mod_titulos_filme2'", '<'), '>')

    def get_title(self):
        self.title = gutils.after(gutils.trim(self.page, "class='mod_titulos_filme'", '<'), '>')

    def get_director(self):
        self.director = gutils.trim(self.page, u'Realização:', '<br>')
        self.director = string.replace(self.director, '&nbsp', '') # website error, ; missing

    def get_plot(self):
        self.plot = gutils.trim(self.page, u'Sinópse:', '</td>')
        self.plot = string.replace(self.plot, u'\x93', '"')
        self.plot = string.replace(self.plot, u'\x94', '"')

    def get_year(self):
        self.year = ''

    def get_runtime(self):
        self.runtime = gutils.clean(gutils.trim(self.page, u'Duração:', '<br'))

    def get_genre(self):
        self.genre = gutils.trim(self.page, 'Categoria:', '<br')

    def get_cast(self):
        self.cast = gutils.clean(gutils.trim(self.page, 'Protagonistas:', '<br'))
        # double replacement
        self.cast = string.replace(self.cast, ', ', '\n')
        self.cast = string.replace(self.cast, ',', '\n')

    def get_classification(self):
        self.classification = gutils.regextrim(self.page, u'Classificação:', '<br>')

    def get_studio(self):
        self.studio = gutils.trim(self.page, 'Distribuidora:', '<br>')

    def get_o_site(self):
        self.o_site = gutils.trim(self.page, 'website.cdv?path=', "'")

    def get_site(self):
        self.site = self.url

    def get_trailer(self):
        self.trailer = 'http://www.clubedevideo.com/mods/mod_trailer.cdv?numero=%s' % self.movie_id

    def get_country(self):
        self.country = ''

    def get_rating(self):
        self.rating = 0
        tmp = gutils.trim(self.page, 'Votação:', '</td>')
        if tmp:
            fullstars = string.split(tmp, 'star_on.gif')
            halfstars = string.split(tmp, 'star_meia')
            rating = 0
            if fullstars:
                rating = len(fullstars) - 1
            if halfstars and len(halfstars) > 1:
                rating = rating + 0.5
            try:
                self.rating = round(rating * 2, 0)
            except:
                pass

    def get_notes(self):
        self.notes = ''

    def get_screenplay(self):
        self.screenplay = ''

    def get_cameraman(self):
        self.cameraman = ''


class SearchPlugin(movie.SearchMovie):

    def __init__(self):
        self.original_url_search = 'http://www.clubedevideo.com/mods/mod_pesquisa_if.cdv?area_pesquisa=nome_original&submitButtonName=++OK++&valor_pesquisa='
        self.translated_url_search = 'http://www.clubedevideo.com/mods/mod_pesquisa_if.cdv?area_pesquisa=nome&submitButtonName=++OK++&valor_pesquisa='
        self.encode = 'iso-8859-1'
        self.remove_accents = False

    def search(self, parent_window):
        self.open_search(parent_window)
        return self.page

    def get_searches(self):
        elements = string.split(self.page, "href='ficha_filme_if.cdv?numero_filme=")
        for element in elements[1:]:
            id = gutils.before(element, "'")
            title = gutils.trim(element, '>', '</a>')
            if id and title:
                self.ids.append(id)
                self.titles.append(title)

#
# Plugin Test
#


class SearchPluginTest(SearchPlugin):
    #
    # Configuration for automated tests:
    # dict { movie_id -> [ expected result count for original url, expected result count for translated url ] }
    #
    test_configuration = {
        'Rocky' : [ 9, 9 ]
    }


class PluginTest:
    #
    # Configuration for automated tests:
    # dict { movie_id -> dict { arribute -> value } }
    #
    # value: * True/False if attribute only should be tested for any value
    #        * or the expected value
    #
    test_configuration = {
        '9384' : {
            'title'               : 'Rocky Balboa',
            'o_title'             : 'Rocky Balboa - 2006',
            'director'            : 'Sylvester Stallone',
            'plot'                : True,
            'cast'                : 'Sylvester Stallone\n\
Burt Young\n\
Tony Burton\n\
MiloVentimiglia',
            'country'             : False,
            'genre'               : 'Acção',
            'classification'      : 'M/12',
            'studio'              : 'Lusomundo',
            'o_site'              : 'http://www.mgm.com/rocky/',
            'site'                : 'http://www.clubedevideo.com/mods/ficha_filme_if.cdv?numero_filme=9384',
            'trailer'             : 'http://www.clubedevideo.com/mods/mod_trailer.cdv?numero=9384',
            'year'                : False,
            'notes'               : False,
            'runtime'             : 102,
            'image'               : True,
            'rating'              : 5,
            'cameraman'           : False,
            'screenplay'          : False
        },
    }



# -*- coding: UTF-8 -*-
# vim: fdm=marker

__revision__ = '$Id: dbupgrade.py 1596 2011-10-04 18:41:03Z piotrek $'

# Copyright (c) 2005-2009 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging
import os.path

from sqlalchemy import *
from sqlalchemy.exc import *

import db
import gutils

log = logging.getLogger("Griffith")


@gutils.popup_message(_('Upgrading database...'))
def upgrade_database(self, version, config):
    """Create new db or update existing one to current format"""
    b = self.engine
    if version == 0 or version is None:
        log.info('Creating new database...')
        # version is 0 or none only for new databases
        db.metadata.create_all(b)
        db.tables.configuration.insert(bind=b).execute(param=u'version', value=unicode(self.version))
        db.tables.media.insert(bind=b).execute(name=u'DVD')
        db.tables.media.insert(bind=b).execute(name=u'DVD-R')
        db.tables.media.insert(bind=b).execute(name=u'DVD-RW')
        db.tables.media.insert(bind=b).execute(name=u'DVD+R')
        db.tables.media.insert(bind=b).execute(name=u'DVD+RW')
        db.tables.media.insert(bind=b).execute(name=u'DVD-RAM')
        db.tables.media.insert(bind=b).execute(name=u'CD')
        db.tables.media.insert(bind=b).execute(name=u'CD-RW')
        db.tables.media.insert(bind=b).execute(name=u'VCD')
        db.tables.media.insert(bind=b).execute(name=u'SVCD')
        db.tables.media.insert(bind=b).execute(name=u'VHS')
        db.tables.media.insert(bind=b).execute(name=u'BETACAM')
        db.tables.media.insert(bind=b).execute(name=u'LaserDisc')
        db.tables.media.insert(bind=b).execute(name=u'HD DVD')
        db.tables.media.insert(bind=b).execute(name=u'Blu-ray')
        db.tables.ratios.insert(bind=b).execute(name=u'16:9')
        db.tables.ratios.insert(bind=b).execute(name=u'16:10')
        db.tables.ratios.insert(bind=b).execute(name=u'4:3')
        db.tables.acodecs.insert(bind=b).execute(name=u'AC-3 Dolby audio')
        db.tables.acodecs.insert(bind=b).execute(name=u'OGG')
        db.tables.acodecs.insert(bind=b).execute(name=u'MP3')
        db.tables.acodecs.insert(bind=b).execute(name=u'MPEG-1')
        db.tables.acodecs.insert(bind=b).execute(name=u'MPEG-2')
        db.tables.acodecs.insert(bind=b).execute(name=u'AAC')
        db.tables.acodecs.insert(bind=b).execute(name=u'Windows Media Audio')
        db.tables.vcodecs.insert(bind=b).execute(name=u'MPEG-1')
        db.tables.vcodecs.insert(bind=b).execute(name=u'MPEG-2')
        db.tables.vcodecs.insert(bind=b).execute(name=u'XviD')
        db.tables.vcodecs.insert(bind=b).execute(name=u'DivX')
        db.tables.vcodecs.insert(bind=b).execute(name=u'H.264')
        db.tables.vcodecs.insert(bind=b).execute(name=u'RealVideo')
        db.tables.vcodecs.insert(bind=b).execute(name=u'QuickTime')
        db.tables.vcodecs.insert(bind=b).execute(name=u'Windows Media Video')
        db.tables.achannels.insert(bind=b).execute(name=u'mono')
        db.tables.achannels.insert(bind=b).execute(name=u'stereo')
        db.tables.achannels.insert(bind=b).execute(name=u'5.1')
        db.tables.achannels.insert(bind=b).execute(name=u'7.1')
        db.tables.subformats.insert(bind=b).execute(name=u'DVD VOB')
        db.tables.subformats.insert(bind=b).execute(name=u'MPL2 (.txt)')
        db.tables.subformats.insert(bind=b).execute(name=u'MicroDVD (.sub)')
        db.tables.subformats.insert(bind=b).execute(name=u'SubRip (.srt)')
        db.tables.subformats.insert(bind=b).execute(name=u'SubViewer2 (.sub)')
        db.tables.subformats.insert(bind=b).execute(name=u'Sub Station Alpha (.ssa)')
        db.tables.subformats.insert(bind=b).execute(name=u'Advanced Sub Station Alpha (.ssa)')
        db.tables.languages.insert(bind=b).execute(name=_('Brazilian Portuguese'))
        db.tables.languages.insert(bind=b).execute(name=_('Bulgarian'))
        db.tables.languages.insert(bind=b).execute(name=_('Catalan'))
        db.tables.languages.insert(bind=b).execute(name=_('Czech'))
        db.tables.languages.insert(bind=b).execute(name=_('Danish'))
        db.tables.languages.insert(bind=b).execute(name=_('Dutch'))
        db.tables.languages.insert(bind=b).execute(name=_('English'))
        db.tables.languages.insert(bind=b).execute(name=_('Estonian'))
        db.tables.languages.insert(bind=b).execute(name=_('French'))
        db.tables.languages.insert(bind=b).execute(name=_('German'))
        db.tables.languages.insert(bind=b).execute(name=_('Greek'))
        db.tables.languages.insert(bind=b).execute(name=_('Hungarian'))
        db.tables.languages.insert(bind=b).execute(name=_('Indonesian'))
        db.tables.languages.insert(bind=b).execute(name=_('Italian'))
        db.tables.languages.insert(bind=b).execute(name=_('Japanese'))
        db.tables.languages.insert(bind=b).execute(name=_('Korean'))
        db.tables.languages.insert(bind=b).execute(name=_('Norwegian Bokmal'))
        db.tables.languages.insert(bind=b).execute(name=_('Occitan'))
        db.tables.languages.insert(bind=b).execute(name=_('Pashto'))
        db.tables.languages.insert(bind=b).execute(name=_('Polish'))
        db.tables.languages.insert(bind=b).execute(name=_('Portuguese'))
        db.tables.languages.insert(bind=b).execute(name=_('Russian'))
        db.tables.languages.insert(bind=b).execute(name=_('Simplified Chinese'))
        db.tables.languages.insert(bind=b).execute(name=_('Slovak'))
        db.tables.languages.insert(bind=b).execute(name=_('Spanish'))
        db.tables.languages.insert(bind=b).execute(name=_('Swedish'))
        db.tables.languages.insert(bind=b).execute(name=_('Turkish'))
        db.tables.tags.insert(bind=b).execute(name=_('Favourite'))
        db.tables.tags.insert(bind=b).execute(name=_('Buy me'))
        return True # upgrade process finished
    #
    # next steps are only for existing databases with an outdated structure
    # not for new created databases
    #
    if version == 1: # fix changes between v1 and v2
        version += 1
        log.info("Upgrading database to version %d...", version)
        b.execute("UPDATE loans SET return_date='2007-01-01' WHERE return_date='None';")
        db_version = self.session.query(db.Configuration).filter_by(param=u'version').one()
        db_version.value = unicode(version)
        self.session.add(db_version)
        self.session.commit()
    if version == 2:    # fix changes between v2 and v3
        #e_type = self.session.bind.engine.dialect.name
        e_type = self.session.bind.name
        version += 1
        log.info("Upgrading database to version %d...", version)

        # create new table
        db.tables.posters.create(checkfirst=True, bind=b)
        db.tables.filters.create(checkfirst=True, bind=b)
        db.tables.ratios.create(checkfirst=True, bind=b)
        try:
            db.tables.ratios.insert(bind=b).execute(name=u'16:9')
            db.tables.ratios.insert(bind=b).execute(name=u'4:3')
        except IntegrityError, e:
            # if the following conversion of the posters takes to long and the user
            # kills the application the database is in a undefined state which throughs that exception
            log.warn("Cannot add values because they exist already: %s", e)

        log.info('... adding new columns')
        # common SQL statements
        queries = {'poster_md5': 'ALTER TABLE movies ADD poster_md5 VARCHAR(32) NULL REFERENCES posters(md5sum);',
                   'ratio_id': 'ALTER TABLE movies ADD ratio_id INTEGER NULL REFERENCES ratios(ratio_id);',
                   'screenplay': 'ALTER TABLE movies ADD screenplay VARCHAR(256) NULL;',
                   'cameraman': 'ALTER TABLE movies ADD cameraman VARCHAR(256) NULL;'}
        # if needed some db specific SQL statements
        if e_type == 'mysql':
            pass
        elif e_type == 'mssql':
            pass
        for key, query in queries.items():
            try:
                self.session.bind.execute(query)
            except OperationalError, e:
                if e.message.lower().find('duplicate column name') > -1:
                    log.warn("Cannot add '%s' column because it exists already: %s", key, e)
                    continue
                else:
                    log.error("Cannot add '%s' column: %s", key, e)
                    return False
            except Exception, e:
                log.error("Cannot add '%s' column: %s", key, e)
                return False

        log.info('... saving posters in database')
        posters_dir = get_old_posters_location(self.data_dir, config, clean_config=True)
        updated = {}
        movies_table = db.metadata.tables['movies']
        for movie in self.session.query(db.Movie.image).all():
            poster_file_name = os.path.join(posters_dir, "%s.jpg" % movie.image)
            if poster_file_name in updated:
                continue
            if os.path.isfile(poster_file_name):
                poster_md5 = gutils.md5sum(file(poster_file_name, 'rb'))
                poster = self.session.query(db.Poster).filter_by(md5sum=poster_md5).first()
                if not poster:
                    poster = db.Poster(md5sum=poster_md5, data=file(poster_file_name, 'rb').read())
                    self.session.add(poster)

                update_query = movies_table.update(movies_table.c.image == movie.image, {'poster_md5': poster_md5, 'image': None}, bind=b)
                # deactivating a 0.12 feature
                save_update_onupdated = movies_table.c['updated'].onupdate
                movies_table.c['updated'].onupdate = None

                try:
                    # yeah, we're commiting inside the loop,
                    # it slows down the process a lot, but at least we can skip buggy posters
                    update_query.execute()
                    self.session.commit()
                except Exception, e:
                    self.session.rollback()
                    log.error(e)
                else:
                    updated[poster_file_name] = True
                    try:
                        os.remove(poster_file_name)
                    except:
                        log.warn("cannot remove %s", poster_file_name)
                finally:
                    movies_table.c['updated'].onupdate = save_update_onupdated
            else:
                log.warn("file not found: %s", movie.image)
                update_query = movies_table.update(movies_table.c.image == movie.image, {'image': None}, bind=b)
                # deactivating a 0.12 feature
                save_update_onupdated = movies_table.c['updated'].onupdate
                movies_table.c['updated'].onupdate = None
                try:
                    update_query.execute()
                finally:
                    movies_table.c['updated'].onupdate = save_update_onupdated
                updated[poster_file_name] = True
        del updated

        db_version = self.session.query(db.Configuration).filter_by(param=u'version').one()
        db_version.value = unicode(version)
        self.session.add(db_version)
        self.session.commit()

    if version == 3:    # fix changes between v3 and v4
        version += 1
        log.info("Upgrading database to version %d...", version)

        log.info('... adding new columns')
        # common SQL statements
        queries = {'barcode': 'ALTER TABLE movies ADD barcode VARCHAR(32) NULL;',
                   'width': 'ALTER TABLE movies ADD width SMALLINT NULL;',
                   'height': 'ALTER TABLE movies ADD height SMALLINT NULL;'}

        for key, query in queries.items():
            try:
                self.session.bind.execute(query)
            except Exception, e:
                log.error("Cannot add '%s' column: %s", key, e)
                return False

        db_version = self.session.query(db.Configuration).filter_by(param=u'version').one()
        db_version.value = unicode(version)
        self.session.add(db_version)
        self.session.commit()

    if version == 4:    # fix changes between v4 and v5
        version += 1
        log.info("Upgrading database to version %d...", version)

        log.info('... deleting old filters')
        # new format, filters were introduced in -beta so we'll free to delete them without warning
        query = 'DELETE FROM filters;'
        self.session.bind.execute(query)

        db_version = self.session.query(db.Configuration).filter_by(param=u'version').one()
        db_version.value = unicode(version)
        self.session.add(db_version)
        self.session.commit()

    if version == 5:    # fix changes between v5 and v6
        version += 1
        log.info("Upgrading database to version %d...", version)

        # common SQL statements
        if self.session.bind.name in ['postgres', 'postgresql']:
            queries = {'created': 'ALTER TABLE movies ADD created TIMESTAMP;',
                       'updated': 'ALTER TABLE movies ADD updated TIMESTAMP;'}
        else:
            queries = {'created': 'ALTER TABLE movies ADD created DATETIME;',
                       'updated': 'ALTER TABLE movies ADD updated DATETIME;'}
        for key, query in queries.items():
            try:
                self.session.bind.execute(query)
            except Exception, e:
                log.error("Cannot add '%s' column: %s", key, e)
                return False

        db_version = self.session.query(db.Configuration).filter_by(param=u'version').one()
        db_version.value = unicode(version)
        self.session.add(db_version)
        self.session.commit()

    return True


# ---------------------------------------------------
# for Griffith <= 0.6.2 compatibility
# ---------------------------------------------------


def convert_from_old_db(config, source_file, destination_file, locations):    #{{{
    """
    convert .gri database into .bd one
    """

    log.info('Converting old database - it can take several minutes...')
    log.debug("Source file: %s", source_file)
    gutils.info(_("Griffith will now convert your database to the new format. This can take several minutes if you have a large database."))
    from sql import GriffithSQL
    from gutils import digits_only
    import os

    if not os.path.isfile(source_file):
        return False

    if 'home' not in locations:
        log.error("locations doesn't contain home path, cannot convert old database")
        return False

    if open(source_file).readline()[:47] == '** This file contains an SQLite 2.1 database **':
        log.debug('SQLite 2.1 detected')
        try:
            import sqlite
        except ImportError:
            log.error('Old DB conversion: please install pysqlite legacy (v1.0)')
            gutils.warning(_("Old DB conversion: please install pysqlite legacy (v1.0)"))
            return False
    else:
        try:    # Python 2.5
            from sqlite3 import dbapi2 as sqlite
        except ImportError: # Python < 2.5 - try to use pysqlite2
            from pysqlite2 import dbapi2 as sqlite

    if os.path.isfile(destination_file):
        # rename destination_file if it already exist
        i = 1
        while True:
            if os.path.isfile("%s_%s" % (destination_file, i)):
                i += 1
            else:
                break
        os.rename(destination_file, "%s_%s" % (destination_file, i))

    try:
        old_db = sqlite.connect(source_file)
    except sqlite.DatabaseError, e:
        if str(e) == 'file is encrypted or is not a database':
            print 'Your database is most probably in SQLite2 format, please convert it to SQLite3:'
            print '$ sqlite ~/.griffith/griffith.gri .dump | sqlite3 ~/.griffith/griffith.gri3'
            print '$ mv ~/.griffith/griffith.gri{,2}'
            print '$ mv ~/.griffith/griffith.gri{3,}'
            print 'or install pysqlite in version 1.0'
            gutils.warning(_("Your database is most probably in SQLite2 format, please convert it to SQLite3"))
        else:
            raise
        return False

    old_cursor = old_db.cursor()

    # fix old database
    old_cursor.execute('PRAGMA encoding = "UTF-8";')
    old_cursor.execute("UPDATE movies SET media = '1' WHERE media = 'DVD';")
    old_cursor.execute("UPDATE movies SET media = '2' WHERE media = 'DVD-R';")
    old_cursor.execute("UPDATE movies SET media = '3' WHERE media = 'DVD-RW';")
    old_cursor.execute("UPDATE movies SET media = '4' WHERE media = 'DVD+R';")
    old_cursor.execute("UPDATE movies SET media = '5' WHERE media = 'DVD+RW';")
    old_cursor.execute("UPDATE movies SET media = '6' WHERE media = 'DVD-RAM';")
    old_cursor.execute("UPDATE movies SET media = '7' WHERE media = 'DivX';")
    old_cursor.execute("UPDATE movies SET media = '7' WHERE media = 'DIVX';")
    old_cursor.execute("UPDATE movies SET media = '7' WHERE media = 'XviD';")
    old_cursor.execute("UPDATE movies SET media = '7' WHERE media = 'XVID';")
    old_cursor.execute("UPDATE movies SET media = '7' WHERE media = 'WMV';")
    old_cursor.execute("UPDATE movies SET media = '9' WHERE media = 'VCD';")
    old_cursor.execute("UPDATE movies SET media = '10' WHERE media = 'SVCD';     ")
    old_cursor.execute("UPDATE movies SET media = '11' WHERE media = 'VHS';")
    old_cursor.execute("UPDATE movies SET media = '12' WHERE media = 'BETACAM';")
    old_cursor.execute("UPDATE movies SET collection_id=0 WHERE collection_id<1")
    old_cursor.execute("UPDATE movies SET volume_id=0 WHERE volume_id<1")
    old_cursor.execute("UPDATE movies SET color=NULL WHERE color<1 OR color='' OR color>3")
    old_cursor.execute("UPDATE movies SET condition=NULL WHERE condition<0 OR condition='' OR condition>5")
    old_cursor.execute("UPDATE movies SET layers=NULL WHERE layers<0 OR layers='' OR layers>4")
    old_cursor.execute("UPDATE movies SET region=NULL WHERE region='' OR region=2 OR region<0 OR region>8")
    old_cursor.execute("UPDATE movies SET year=NULL WHERE year<1900 or year>2007")
    old_cursor.execute("UPDATE movies SET rating = 0 WHERE rating NOT IN (0,1,2,3,4,5,6,7,8,9,10);") # rating>10 doesn't work with some DB
    old_cursor.execute("UPDATE movies SET runtime = NULL WHERE runtime > 10000;") # remove strings
    old_cursor.execute("UPDATE loans SET return_date=NULL WHERE return_date=''")
    old_cursor.execute("UPDATE loans SET return_date=NULL WHERE return_date='None'")
    old_cursor.execute("DELETE FROM loans WHERE date='' OR date ISNULL")
    old_cursor.execute("DELETE FROM volumes WHERE name = ''")
    old_cursor.execute("DELETE FROM volumes WHERE name = 'None'")
    old_cursor.execute("DELETE FROM collections WHERE name = ''")
    old_cursor.execute("DELETE FROM collections WHERE name = 'None'")
    old_cursor.execute("DELETE FROM languages WHERE name = ''")

    config.set('type', 'sqlite', section='database')
    config.set('file', 'griffith.db', section='database')
    config['posters'] = 'posters'
    config.set('color', 0, section='defaults')
    config.set('condition', 0, section='defaults')
    config.set('layers', 0, section='defaults')
    config.set('media', 0, section='defaults')
    config.set('region', 0, section='defaults')
    config.set('vcodec', 0, section='defaults')
    locations['posters'] = os.path.join(locations['home'], 'posters')
    new_db = GriffithSQL(config, locations['home'], fallback=False)

    # collections
    collection_mapper = {'': None, u'': None, 0: None, '0': None, -1: None, '-1': None}
    old_cursor.execute("SELECT id, name FROM collections;") # loaned status will be set later - buggy databases :-(
    for i in old_cursor.fetchall():
        o = db.Collection(name=i[1])
        try:
            new_db.session.add(o)
            new_db.session.commit()
        except Exception, e:
            log.error(e)
            continue
        collection_mapper[i[0]] = o.collection_id

    # volumes
    volume_mapper = {'': None, u'': None, 0: None, '0': None, -1: None, '-1': None}
    old_cursor.execute("SELECT id, name FROM volumes;") # loaned status will be set later - buggy databases :-(
    for i in old_cursor.fetchall():
        o = db.Volume(name=i[1])
        try:
            new_db.session.add(o)
            new_db.session.commit()
        except Exception, e:
            log.error(e)
            continue
        volume_mapper[i[0]] = o.volume_id

    # people
    person_mapper = {}
    old_cursor.execute("SELECT id, name, email, phone FROM people;")
    for i in old_cursor.fetchall():
        o = db.Person(name=i[1], email=i[2], phone=i[3])
        try:
            new_db.session.add(o)
            new_db.session.commit()
        except Exception, e:
            log.error(e)
            continue
        person_mapper[i[0]] = o.person_id

    # languages
    language_mapper = {'': None, u'': None, 0: None, '0': None, -1: None, '-1': None}
    old_cursor.execute("SELECT id, name FROM languages;")
    for i in old_cursor.fetchall():
        o = new_db.session.query(db.Lang).filter_by(name=i[1]).first()
        if o is not None:
            language_mapper[i[0]] = o.lang_id
        else:
            o = db.Lang(name=i[1])
            try:
                new_db.session.add(o)
                new_db.session.commit()
            except Exception, e:
                log.error(e)
                continue
            language_mapper[i[0]] = o.lang_id

    # media
    medium_mapper = {'': None, u'': None, 0: None, '0': None, -1: None, '-1': None}
    old_cursor.execute("SELECT id, name FROM media;")
    for i in old_cursor.fetchall():
        o = new_db.session.query(db.Medium).filter_by(name=i[1]).first()
        if o is not None:
            medium_mapper[i[0]] = o.medium_id
        else:
            o = db.Medium(name=i[1])
            try:
                new_db.session.add(o)
                new_db.session.commit()
            except Exception, e:
                log.error(e)
                continue
            medium_mapper[i[0]] = o.medium_id

    # tags
    tag_mapper = {}
    old_cursor.execute("SELECT id, name FROM tags;")
    for i in old_cursor.fetchall():
        o = new_db.session.query(db.Tag).filter_by(name=i[1]).first()
        if o is not None:
            tag_mapper[i[0]] = o.tag_id
        else:
            o = db.Tag(name=i[1])
            try:
                new_db.session.add(o)
                new_db.session.commit()
            except Exception, e:
                log.error(e)
                continue
            tag_mapper[i[0]] = o.tag_id

    # movies
    movie_mapper = {}
    old_cursor.execute("""
        SELECT id, volume_id, collection_id, original_title, title, director,
            number, image, plot, country, year, runtime, classification,
            genre, studio, site, imdb, actors, trailer, rating, loaned,
            media, num_media, obs, seen, region, condition, color, layers
        FROM movies ORDER BY number;""")
    for i in old_cursor.fetchall():
        o = db.Movie()
        o.number = digits_only(i[6])
        if i[1] in volume_mapper:
            o.volume_id = volume_mapper[i[1]]
        if i[2] in collection_mapper:
            o.collection_id = collection_mapper[i[2]]
        o.o_title = i[3][:255]
        o.title = i[4][:255]
        o.director = i[5][:255]
        o.image = i[7][:128]
        o.plot = i[8]
        o.country = i[9][:128]
        o.year = digits_only(i[10])
        o.runtime = digits_only(i[11])
        o.classification = i[12][:128]
        o.genre = i[13][:128]
        o.studio = i[14][:128]
        o.o_site = i[15][:255]
        o.site = i[16][:255]
        o.cast = i[17]
        o.trailer = i[18][:255]
        o.rating = digits_only(i[19])
        #o.loaned = bool(i[20]) # updated later
        if int(i[21]) in medium_mapper:
            o.medium_id = medium_mapper[int(i[21])]
        o.media_num = digits_only(i[22])
        o.notes = i[23]
        o.seen = bool(i[24])
        o.region = digits_only(i[25])
        o.cond = digits_only(i[26], 5)
        o.color = digits_only(i[27], 3)
        o.layers = digits_only(i[28], 4)

        try:
            new_db.session.add(o)
            new_db.session.commit()
        except Exception, e:
            log.error(e)
            continue
        movie_mapper[i[0]] = o.movie_id

    # movie tag
    old_cursor.execute("SELECT movie_id, tag_id FROM movie_tag WHERE movie_id IN (SELECT id FROM movies);")
    for i in old_cursor.fetchall():
        o = new_db.session.query(db.MovieTag).filter_by(movie_id=movie_mapper[i[0]], tag_id=tag_mapper[i[1]]).first()
        if o is None:
            m = new_db.session.query(db.Movie).filter_by(movie_id=movie_mapper[i[0]]).one()
            t = new_db.session.query(db.Tag).filter_by(tag_id=tag_mapper[i[1]]).one()
            m.tags.append(t)
            try:
                new_db.session.add(m)
                new_db.session.commit()
            except Exception, e:
                log.error(e)
                continue

    # movie lang
    old_cursor.execute("SELECT movie_id, lang_id, type FROM movie_lang WHERE movie_id IN (SELECT id FROM movies);")
    for i in old_cursor.fetchall():
        o = new_db.session.query(db.MovieLang).filter_by(movie_id=movie_mapper[i[0]], lang_id=language_mapper[i[1]], type=i[2]).first()
        if o is None:
            m = new_db.session.query(db.Movie).filter_by(movie_id=movie_mapper[i[0]]).one()
            l = db.MovieLang(lang_id=language_mapper[i[1]], type=i[2])
            m.languages.append(l)
            try:
                new_db.session.add(m)
                new_db.session.commit()
            except Exception, e:
                log.error(e)
                continue

    # loans
    old_cursor.execute("SELECT person_id, movie_id, volume_id, collection_id, date, return_date FROM loans;")
    for i in old_cursor.fetchall():
        vol = col = None
        not_returned = i[5] is None

        if int(i[2]) > 0:
            try:
                vol = new_db.session.query(db.Volume).filter_by(volume_id=volume_mapper[i[2]]).one()
            except Exception, e:
                log.error(e)
                continue
        if int(i[3]) > 0:
            try:
                col = new_db.session.query(db.Collection).filter_by(collection_id=collection_mapper[i[3]]).one()
            except Exception, e:
                log.error(e)
                continue
        if int(i[1]) == 0:
            if vol is not None and len(vol.movies) > 0:
                m = vol.movies[0]
            elif col is not None and len(col.movies) > 0:
                m = col.movies[0]
            else:
                log.warn("Cannot find associated movie for this loan (%s)" % i)
                continue
        else:
            try:
                m = new_db.session.query(db.Movie).filter_by(movie_id=movie_mapper[i[1]]).one()
            except Exception, e:
                log.error(e)
                continue

        l = db.Loan()
        l.person_id = person_mapper[i[0]]
        l.date = str(i[4])[:10]
        if not_returned:
            m.loaned = True
            l.return_date = None
        else:
            l.return_date = str(i[5])[:10]

        # update volume / collection status
        if int(i[2]) > 0:
            l.volume_id = volume_mapper[i[2]]
            if not_returned:
                vol.loaned = True
                vol.save()
        if int(i[3]) > 0:
            l.collection_id = collection_mapper[i[3]]
            if not_returned:
                col.loaned = True
                col.save()
        l.save()
        m.loans.append(l)
        try:
            new_db.session.add(m)
            new_db.session.commit()
        except Exception, e:
            log.error(e)
            continue
    #clear_mappers()
    return new_db
#}}}


def get_old_posters_location(home_dir, config, clean_config=False):
    if config.get('posters', None) is not None:
        dirname = config.get('posters')
    elif config.get('type', 'sqlite', section='database') == 'sqlite':
        dbname = config.get('name', 'griffith', section='database')
        if dbname != 'griffith':
            dirname = "posters_sqlite_%s" % dbname
        else:
            dirname = 'posters'
    else:
        dirname = "posters_%(type)s_%(host)s_%(port)s_%(name)s_%(user)s" % config.to_dict('database')
    if clean_config:
        config.remove_option('posters')
        config.save()
    return os.path.join(home_dir, dirname)



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2009 Michael Jahn
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import xml.dom.minidom
import datetime
from plugins.export import XmlExportBase

class ExportPlugin(XmlExportBase):
    plugin_name         = "Ant Movie Catalog Database XML Export"
    plugin_description  = _("Full Ant Movie Catalog database xml export plugin")
    plugin_author       = "Michael Jahn"
    plugin_author_email = "griffith@griffith.cc"
    plugin_version      = "1.0"

    def __init__(self, database, locations, parent_window, search_conditions, config):
        XmlExportBase.__init__(self, database, locations, parent_window, search_conditions, config)
        self.config_section = 'export-antmovie'
        self.export_name    = 'Ant Movie Catalog database export'
        self.filename       = 'ant-movie-catalog.xml'
        self.encoding       = 'iso-8859-1'

    def export_to_document(self, document, mainelement):
        collelement = document.createElement('AntMovieCatalog')
        collelement.setAttribute('Format', '35')
        collelement.setAttribute('Version', '3.5.1 (2007-09-22)')
        collelement.setAttribute('Date', datetime.datetime.now().strftime('%d.%m.%Y %H:%M:%S'))
        document.appendChild(collelement)
        comment = document.createComment('Griffith Export in Ant Movie Catalog database format')
        document.insertBefore(comment, collelement)
        catalogelement = document.createElement('Catalog')
        collelement.appendChild(catalogelement)
        propertieselement = document.createElement('Properties')
        catalogelement.appendChild(propertieselement)
        contentselement = document.createElement('Contents')
        catalogelement.appendChild(contentselement)
        XmlExportBase.export_to_document(self, document, contentselement)

    def process_movie(self, document, mainelement, movie, keys):
        # create node
        itemelement = document.createElement('Movie')
        itemelement.setAttribute('Checked', 'False')
        mainelement.appendChild(itemelement)
        XmlExportBase.process_movie(self, document, itemelement, movie, keys)

    def process_movie_value(self, document, dvdelement, key, value):
        if key == 'movies_number':
            dvdelement.setAttribute('Number', value)
        elif key == 'movies_loaned':
            pass
        elif key == 'movies_seen':
            dvdelement.setAttribute('Checked', value)
        elif key == 'movies_rating':
            dvdelement.setAttribute('Rating', value)
        elif key == 'movies_color':             # SmallInteger
            pass
        elif key == 'movies_cond':              # SmallInteger
            pass
        elif key == 'movies_layers':            # SmallInteger
            pass
        elif key == 'movies_media_num':         # SmallInteger
            dvdelement.setAttribute('Disks', value)
        elif key == 'movies_runtime':           # Integer
            dvdelement.setAttribute('Length', value)
        elif key == 'movies_year':              # year
            dvdelement.setAttribute('Year', value)
        elif key == 'movies_o_title':           # VARCHAR
            dvdelement.setAttribute('OriginalTitle', value)
            dvdelement.setAttribute('FormattedTitle', value)
        elif key == 'movies_title':             # VARCHAR
            dvdelement.setAttribute('TranslatedTitle', value)
        elif key == 'movies_director':          # VARCHAR
            dvdelement.setAttribute('Director', value)
        elif key == 'movies_o_site':            # VARCHAR
            pass
        elif key == 'movies_site':              # VARCHAR
            dvdelement.setAttribute('URL', value)
        elif key == 'movies_trailer':           # VARCHAR
            pass
        elif key == 'movies_country':           # VARCHAR
            dvdelement.setAttribute('Country', value)
        elif key == 'movies_genre':             # VARCHAR
            dvdelement.setAttribute('Category', value.replace('\n', ''))
        elif key == 'movies_region':            # SmallInteger
            pass
        elif key == 'movies_studio':            # VARCHAR
            pass
        elif key == 'movies_classification':    # VARCHAR
            pass
        elif key == 'movies_cast':              # VARCHAR
            if len(value.strip()) > 0:
                actors = value.split('\n')
                actorsstring = ''
                delim = ''
                for actor in actors:
                    actorsplit = actor.split(_(' as '))
                    actornames = actorsplit[0]
                    if len(actorsplit) > 1:
                        actorsstring += delim + actornames.strip() + ' (' + actorsplit[1].strip() + ')'
                    else:
                        actorsstring += delim + actornames.strip()
                    delim = ', '
                dvdelement.setAttribute('Actors', actorsstring)
        elif key == 'movies_plot':              # VARCHAR
            dvdelement.setAttribute('Description', value.replace('\n', '|'))
        elif key == 'movies_notes':             # VARCHAR
            dvdelement.setAttribute('Comments', value.replace('\n', '|'))
        elif key == 'vcodecs_name':             # VARCHAR
            dvdelement.setAttribute('VideoFormat', value)

    def process_movie_image(self, document, dvdelement, imagedata, imagemd5sum):
        if imagemd5sum and imagedata:
            dvdelement.setAttribute('Picture', self.filename + '_pictures/' + imagemd5sum + '.jpg')
            self.save_image_to_file(imagedata, imagemd5sum, self.filename + '_pictures/')

    def process_movie_tags(self, document, movieelement, movie):
        pass

    def process_acodecs(self, document, movieelement, acodecsquery):
        acodecsstring = ''
        delim = ''
        codecsused = []
        for acodec in acodecsquery.execute().fetchall():
            if acodec['name'] not in codecsused:
                acodecsstring += delim + self.convert_value(acodec['name'])
                delim = ', '
                codecsused.append(acodec['name'])
        movieelement.setAttribute('AudioFormat', acodecsstring)
        
    def process_languages(self, document, movieelement, languagesquery):
        languagesstring = ''
        delim = ''
        languagesused = []
        for language in languagesquery.execute().fetchall():
            if language['languages_name'] not in languagesused:
                languagesstring += delim + self.convert_value(language['languages_name'])
                delim = ', '
                languagesused.append(language['languages_name'])
        movieelement.setAttribute('Languages', languagesstring)

    def process_subtitles(self, document, movieelement, subtitlesquery):
        subtitlesstring = ''
        delim = ''
        for subtitle in subtitlesquery.execute().fetchall():
            subtitlesstring += delim + self.convert_value(subtitle['name'])
            delim = ', '
        movieelement.setAttribute('Subtitles', subtitlesstring)
